// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include<cpgplot.h>
#include<cmath>
#include"graph.h"
#include<iostream>
#include<vector>
#include <algorithm>
#include<numeric>


/****************************************************************
 *
 *
 *****************************************************************/

class alist
{
public:
    float x;
    float y;
public:
    alist(float a, float b) : x(a), y(b) {};
    alist(const alist &a)
    {
      x = a.x;
      y = a.y;
    }
    ~alist() {};



  bool  operator<(const alist &a) const 
  {

    cout << "alist sorting x=" << x << " a.x=" << a.x << endl;
    if (x < a.x) return true;
    if (x > a.x) return false;
    return true;

  };
};



struct aListSort
{

  bool operator()(alist*& a, alist*& b)
  {  
  if (a->x < b->x) return true;
  if (a->x > b->x) return false;
  return true;
  }


};




float HistogramMaximum(int maxFreq, int nData, float &yMax1);
float* TransformMatrix(float xmin, float xbinWidth, 
		       float ymin, float ybinWidth);



void loadDensity( float *a, int nx, int ny, 
                  float xmin, float xmax, 
                  float ymin, float ymax, 
                  vector<float> & xdata, vector<float> & ydata, int ndata, float &fMax);



void SetUpPalette(int type);
void DrawColourScale(char side[], float disp, float width, char *label, 
		     float foreGround, float backGround);

void SetUpColourIndices( bool display = false );
void SetUpDefaultColourIndices( );



void drawGraph( int ndata, 
                float* xdata, 
                float *xerrors, 
                float *ydata, 
                float *yerrors,
                char *title, 
                char *xlabel, 
                char *ylabel, 
                float x1, 
                float x2,
                float y1, 
                float y2,             
                float xmin1, 
                float xmax1, 
                float ymin1, 
                float ymax1, 
                bool equalAxes)
  
{
    
    float *lower = 0, *upper = 0;
    float xmin, xmax, ymin, ymax;
  
       
    xmin =  10000.0;
    xmax = -10000.0;
    ymin = xmin;
    ymax = xmax;
             
    for (int j = 0; j <  ndata; j++)
    {
        
        if (xdata[j] < xmin) xmin = xdata[j];
        if (xdata[j] >  xmax) xmax = xdata[j];
        if (ydata[j] < ymin) ymin = ydata[j];
        if (ydata[j] > ymax) ymax = ydata[j];
    }
				  
    if (fabs(ymax - ymin) < 1e-10) 
    {
        ymin = ymin - 0.1;
        ymax = ymax + 0.1;
    }    

    cpgsci( 1 );
    cpgsvp(x1, x2, y1, y2);

     
    cpgrnge(xmin, xmax, &xmin, &xmax);
    cpgrnge(ymin, ymax, &ymin, &ymax);



    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }
  
    if ( fabs(ymin1 - ymax1) > 1e-6) 
    {
        ymin = ymin1;
        ymax = ymax1;
    }
  



    
    if (equalAxes)
    { 
	 cpgwnad(xmin, xmax, ymin, ymax);
    }
    else
    {
        cpgswin(xmin, xmax, ymin, ymax);
        
    }
    cpgsch(1.0);
      
  
    cpgsls(1);
    cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
    cpgsch(0.8);
    cpgslw(1);
    cpgbox("n", 0.0, 0, "n", 0.0, 0);
   





    cpgsch(1.0); 
    cpgslw(1);
    cpglab("", ylabel, title);
    cpgmtxt("B", 2.0, 0.5, 0.5, xlabel );
  

    cpgslw(1);
  
    cpgsch(0.6);        
    cpgpt(ndata, xdata, ydata, 9);
   

       
    lower = new float[ndata];
    upper = new float[ndata];
    
    for (int i = 0; i < ndata; i++)
    { 
        lower[i] = xdata[i] - xerrors[i];
        upper[i] = xdata[i] + xerrors[i];
    }       
    cpgerrx(ndata, lower, upper, ydata, 0);
          

    for (int i = 0; i < ndata; i++)
    { 
        lower[i] = ydata[i] - yerrors[i];
        upper[i] = ydata[i] + yerrors[i];
    }       

           
    cpgerry(ndata, xdata, lower, upper, 0);
    delete [] lower;
    delete [] upper;  

    cpgslw(1);
    cpgsch(0.5);
 
}

/**********************************************************************
*
**********************************************************************/


void drawGraph(
                vector<float> &xdata  , 
                vector<float> &xerrors  , 
                vector<float> &ydata  , 
                vector<float> &yerrors  ,
                vector<int>   &colourIndex,
                char *title, 
                char *xlabel, 
                char *ylabel, 
                float x1, 
                float x2,
                float y1, 
                float y2,             
                float xmin1, 
                float xmax1, 
                float ymin1, 
                float ymax1, 
                bool equalAxes)
  
{
    
    int ndata = xdata.size();


    cout << "graph line 203 ndata=" << ndata << endl;

    float *lower = 0, *upper = 0;
    float xmin, xmax, ymin, ymax;
  
       
    xmin =  10000.0;
    xmax = -10000.0;
    ymin = xmin;
    ymax = xmax;
             
  
  


    for (int j = 0; j <  ndata; j++)
    {
     


        if (xdata[j] < xmin) xmin = xdata[j];
        if (xdata[j] >  xmax) xmax = xdata[j];
        if (ydata[j] < ymin) ymin = ydata[j];
        if (ydata[j] > ymax) ymax = ydata[j];
    }
				  
    if (fabs(ymax - ymin) < 1e-10) 
    {
        ymin = ymin - 0.1;
        ymax = ymax + 0.1;
    }    

    cpgsci( 1 );
    cpgsvp(x1, x2, y1, y2);

     
    cpgrnge(xmin, xmax, &xmin, &xmax);
    cpgrnge(ymin, ymax, &ymin, &ymax);



    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }
  
    if ( fabs(ymin1 - ymax1) > 1e-6) 
    {
        ymin = ymin1;
        ymax = ymax1;
    }
  



    
    if (equalAxes)
    { 
	 cpgwnad(xmin, xmax, ymin, ymax);
    }
    else
    {
        cpgswin(xmin, xmax, ymin, ymax);
        
    }
    cpgsch(1.0);
      
  
    cpgsls(1);
    cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
    cpgsch(0.8);
    cpgslw(1);
    cpgbox("n", 0.0, 0, "n", 0.0, 0);
   





    cpgsch(1.0); 
    cpgslw(1);
    cpglab("", ylabel, title);
    cpgmtxt("B", 2.0, 0.5, 0.5, xlabel );
  

    cpgslw(1);
  
    cpgsch(0.6);        


     

       
    lower = new float[ndata];
    upper = new float[ndata];
    
    for (int i = 0; i < ndata; i++)
    { 
        cpgsci( colourIndex[i] ); 
        cpgpt1( xdata[i], ydata[i], 9);

   
   
        cpgerr1(1, xdata[i], ydata[i], xerrors[i], 0);
       
        cpgerr1(3, xdata[i], ydata[i], xerrors[i], 0);
   
        cpgerr1(2, xdata[i], ydata[i], yerrors[i], 0);
       
        cpgerr1(4, xdata[i], ydata[i], yerrors[i], 0);
   

    }       

           
   
 

    cpgslw(1);
    cpgsch(0.5);
 


     
  
 



}





/**********************************************************************
*
**********************************************************************/
void drawMap( vector<float> &xdata  ,          
              vector<float> &ydata  ,      
              char *title, 
              char *xlabel, 
              char *ylabel, 
              float x1, 
              float x2,
              float y1, 
              float y2,             
              float xmin1, 
              float xmax1, 
              float ymin1, 
              float ymax1, 
              bool equalAxes)
  
{
    
    int ndata = xdata.size();


    cout << "drawmap line 203 ndata=" << ndata << endl;

    if (ndata < 2) return;

    SetUpDefaultColourIndices( );

    cpgsci(1);



    float *lower = 0, *upper = 0;
    float xmin, xmax, ymin, ymax;
  
       
    xmin =  10000.0;
    xmax = -10000.0;
    ymin = xmin;
    ymax = xmax;
             
  
 
    for (int j = 0; j <  ndata; j++)
    {
        if (xdata[j] < xmin) xmin = xdata[j];
        if (xdata[j] >  xmax) xmax = xdata[j];
        if (ydata[j] < ymin) ymin = ydata[j];
        if (ydata[j] > ymax) ymax = ydata[j];
    }
				  
    if (fabs(ymax - ymin) < 1e-10) 
    {
        ymin = ymin - 0.1;
        ymax = ymax + 0.1;
    }    

  
    cpgsvp(x1, x2, y1, y2);

     
    cpgrnge(xmin, xmax, &xmin, &xmax);
    cpgrnge(ymin, ymax, &ymin, &ymax);



    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }
  
    if ( fabs(ymin1 - ymax1) > 1e-6) 
    {
        ymin = ymin1;
        ymax = ymax1;
    }
  


    
    if (equalAxes)
    { 
	 cpgwnad(xmin, xmax, ymin, ymax);
    }
    else
    {
        cpgswin(xmin, xmax, ymin, ymax);
        
    }
    cpgsch(1.0);
      
  
    cpgsls(1);
    cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
    cpgsch(0.8);
    cpgslw(1);
    cpgbox("n", 0.0, 0, "n", 0.0, 0);
   



    cpgsch(1.0); 
    cpgslw(1);
    cpglab("", ylabel, title);
    cpgmtxt("B", 2.0, 0.5, 0.5, xlabel );
  

    cpgslw(1);
  
    cpgsch(0.6);        

    
       
    lower = new float[ndata];
    upper = new float[ndata];
    
    for (int i = 0; i < ndata; i++)
    { 
    
      //        cpgpt1( xdata[i], ydata[i], 9);

   
   


    }       
 


    int nx = 100, ny = 100;


    float dx = (xmax - xmin) / (float)nx;
    float dy = (ymax - ymin) / (float)ny;


    cout << "drawmap line 496\n";
          

    float* tr = TransformMatrix( xmin, dx, ymin, dy); 
               





    const int noContours = 99;
    float *c = new float;

    char label[noContours][10];
    for(int i = 0; i < noContours; i ++)
    {
      sprintf(label[i], "%4.1f", c[i]);

    }
    float *a = new float[nx * ny];

  
          
    float fMax = 0.0;
    loadDensity( a, nx, ny, xmin, xmax, ymin, ymax, xdata, ydata, ndata, fMax);

    delete [] c;

    cout << "drawmap line 526\n";




    //    for (int i = 0; i < nx; i++)
    //        for (int j = 0; j < ny; j++)
    //	  a[j * ny + i] = 0.0;

    //  for (int i = 0; i < nx; i++)
    //    a[i + ny * nx / 2] = 5.0;



    //  for (int j = 0; j < ny; j++)
    //    a[ j * ny  + nx / 2] = 8.0;



    //    cpgcont( a, nx, ny, 1, nx, 1, ny, c, 20, tr);
 



     cpgsch(0.4);
     for(int i = 0; i < 20; i++)
     {  
       //         cpgsci(i + 1);
	 //         cpgconl( a, nx, ny, 1, nx, 1, ny, c[i], tr, label[i], 50, 20);
     }

  


    cout << "drawmap line 561\n";



    SetUpPalette( 3 );
    cpgsitf( 2 );




    float xl = 0.01, xu = fMax; 

 
    cpgimag( a, nx, ny, 1, nx, 1, ny, xl, xu, tr );


    cout << "drawmap line 567\n";


    /***************************
    * Draw the colour-scale bar.
    ***************************/

    cpgsci( 10 );
    cpgstbg( 0 );

  
    DrawColourScale("RI", 0.75, 2.5, "PIXEL COUNT", xl, xu );



    SetUpDefaultColourIndices();

   cpgsci( 14 );



    cpgsch(1.0);
      
  
    cpgsls(1);
    cpgbox("bctsig", 0.0, 0, "bctsig", 0.0, 0);
   

    cpgsci( 1 );

    cpgsch(0.7);
    char text[100];
    sprintf(text, "Maximum fequency=%d (No of points=%d)", (int)fMax, ndata);
    cpgmtxt("T", 1.0, 0.0, 0.0, text );


    delete [] tr;
    delete [] a;
    cpgslw(1);
    cpgsch(0.5);
   

}













/****************************************************************
*
*
*****************************************************************/
void drawGraph( int ndata, 
                float* xdata, 
                float *ydata, 
                char *title, 
                char *xlabel, 
                char *ylabel, 
                float x1, 
                float x2, 
                float y1, 
                float y2, 
                float xmin1, 
                float xmax1, 
                float ymin1, 
                float ymax1, 
                bool equalAxes)
  
{
    

  cout << "Drawgraph line 657 xmin1=" << xmin1 << " xmax1=" << xmax1 << " ymin1=" << ymin1 << " ymax1=" << ymax1 << endl;
  
       
    float xmin =  10000.0;
    float xmax = -10000.0;
    float ymin = xmin;
    float ymax = xmax;
             
    for (int j = 0; j <  ndata; j++)
    {
        
        if (xdata[j] < xmin) xmin = xdata[j];
        if (xdata[j] > xmax) xmax = xdata[j];
        if (ydata[j] < ymin) ymin = ydata[j];
        if (ydata[j] > ymax) ymax = ydata[j];
    }
				  
    if (fabs(ymax - ymin) < 1e-10) 
    {
        ymin = ymin - 0.1;
        ymax = ymax + 0.1;
    }    

    cpgsci( 1 );
    cpgsvp(x1, x2, y1, y2);

     
    cpgrnge(xmin, xmax, &xmin, &xmax);
    cpgrnge(ymin, ymax, &ymin, &ymax);



    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }
  
    if ( fabs(ymin1 - ymax1) > 1e-6) 
    {
        ymin = ymin1;
        ymax = ymax1;
    }
  

    cout << "draw graph xmin1=" << xmin1 << " xmax1=" << xmax1 << " ymin1=" << ymin1 << " ymax1=" << ymax1 << endl;
  cout << "draw graph xmin=" << xmin << " xmax=" << xmax << " ymin=" << ymin << " ymax=" << ymax << endl;



    
    if (equalAxes)
    { 
	 cpgwnad(xmin, xmax, ymin, ymax);
    }
    else
    {
        cpgswin(xmin, xmax, ymin, ymax);
        
    }
    cpgsch(1.0);
      
  
    cpgsls(1);
    cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
    cpgsch(0.8);
    cpgslw(1);
    cpgbox("n", 0.0, 0, "n", 0.0, 0);
   





    cpgsch(1.0); 
    cpgslw(1);
    cpglab("", ylabel, title);
    cpgmtxt("B", 2.0, 0.5, 0.5, xlabel );
  

    cpgslw(1);
  
    cpgsch(0.6);        
    cpgpt(ndata, xdata, ydata, 9);
   


    cpgslw(1);
    cpgsch(0.5);
 
}





/****************************************************************
*
*
*****************************************************************/
void drawGraph( 
                vector<float> &xdataIn, 
                vector<float> &ydataIn, 
              
                char *title, 
                char *xlabel, 
                char *ylabel, 
                float x1, 
                float x2, 
                float y1, 
                float y2, 
                float xmin1, 
                float xmax1, 
                float ymin1, 
                float ymax1, 
                bool equalAxes)
  
{
    
    int ndata = xdataIn.size();
    float *xdata = new float[ndata];
    float *ydata = new float[ndata];

    float xmin, xmax, ymin, ymax;
  
       
    xmin =  10000.0;
    xmax = -10000.0;
    ymin = xmin;
    ymax = xmax;
             
    for (int j = 0; j <  ndata; j++)
    {
        xdata[j] = xdataIn[j];
        ydata[j] = ydataIn[j];

        if (xdata[j] < xmin) xmin = xdata[j];
        if (xdata[j] > xmax) xmax = xdata[j];
        if (ydata[j] < ymin) ymin = ydata[j];
        if (ydata[j] > ymax) ymax = ydata[j];
    }
				  
    if (fabs(ymax - ymin) < 1e-10) 
    {
        ymin = ymin - 0.1;
        ymax = ymax + 0.1;
    }    

    cpgsci( 1 );
    cpgsvp(x1, x2, y1, y2);

     
    cpgrnge(xmin, xmax, &xmin, &xmax);
    cpgrnge(ymin, ymax, &ymin, &ymax);



    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }
  
    if ( fabs(ymin1 - ymax1) > 1e-6) 
    {
        ymin = ymin1;
        ymax = ymax1;
    }
  



    
    if (equalAxes)
    { 
	 cpgwnad(xmin, xmax, ymin, ymax);
    }
    else
    {
        cpgswin(xmin, xmax, ymin, ymax);
        
    }
    cpgsch(1.0);
      




  
    cpgsls(1);
    cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
    cpgsch(0.8);
    cpgslw(1);
    cpgbox("n", 0.0, 0, "n", 0.0, 0);
   





    cpgsch(1.0); 
    cpgslw(1);
    cpglab("", ylabel, title);
    cpgmtxt("B", 2.0, 0.5, 0.5, xlabel );
  

    cpgslw(1);
  
    cpgsch(0.6);        
    cpgpt(ndata, xdata, ydata, 9);
   


    cpgslw(1);
    cpgsch(0.5);

    delete [] xdata;
    delete [] ydata;

}






/****************************************************************
 *
 *
 *****************************************************************/
void drawGraph( int ngraphs, 
                vector< vector<float> > &xdata,
                vector< vector<float> > &ydata, 
                char *xlabel, 
                char *ylabel, 
                char *title,                 
                vector<string> &subTitle,            
                float x1, 
                float x2,
                float y1,
                float y2,  
                float xmin1, 
                float xmax1, 
                float ymin1, 
                float ymax1) 
               
  
{
    
    

       
    float xmin =  1000000000.0;
    float xmax = -1000000000.0;
    float ymin =  1000000000.0;
    float ymax = -1000000000.0;


    cout << "drawgraph line 334\n";


    for (int i = 0; i < ngraphs; i++)             
    {
      cout << "daya for graph " << i << endl;
        for (unsigned j = 0; j <  xdata[i].size(); j++)
        {
        
            if (xdata[i][j] < xmin) xmin = xdata[i][j];
            if (xdata[i][j] >  xmax) xmax = xdata[i][j];
            if (ydata[i][j] < ymin) ymin = ydata[i][j];
            if (ydata[i][j] >  ymax) ymax = ydata[i][j];
	    //            cout << j << " x=" << xdata[i][j] << " y=" << ydata[i][j] << endl;


        }
    }				  

    cout << "Line 352 xmin=" << xmin << " xmax=" << xmax << " ymin=" << ymin << " ymax=" << ymax << endl;


    cpgrnge(xmin, xmax, &xmin, &xmax);





 

    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }

  
    if ( fabs(ymin1 - ymax1) > 1e-6) 
    {
        ymin = ymin1;
        ymax = ymax1;
    }

  


    cout << "Line 379 xmin=" << xmin << " xmax=" << xmax << " ymin=" << ymin << " ymax=" << ymax << endl;




    cout << "graph.cc line 380 title=" << title << endl;
    cout << "ylabel=" << ylabel << endl;

    cout << "x1=" << x1 << " x2=" << x2 << " y1=" << y1 << " y2=" << y2 << endl;
    cout << "xmin=" << xmin << " xmax=" << xmax << " ymin=" << ymin << " ymax=" << ymax << endl;


    cpgsvp(x1, x2, y1, y2);

    cpgswin(xmin, xmax, ymin, ymax);


    
    cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
    cpgbox("n", 0.0, 0, "n", 0.0, 0);
    cpgsch(1.0); 
    cpgslw(1);
    cpglab("", ylabel, "");
    cpgmtxt("B", 2.0, 0.5, 0.5, xlabel ); 
    cpgmtxt("T", 1.0, 0.5, 0.5, title );


    
    for (int i = 0; i < ngraphs; i++)                 
    {

        int nData = xdata[i].size();           

        if (nData == 0) continue;
	cout << "Line 407 drawing graph " << i << endl;
         

        cpgsci(i + 1); 
     

        cpgsls(1);


      
        cpgslw(1);
  

        cpgsch(0.7);
      
        cpgmtxt("T", -(i + 1), 0.0, 0.0, subTitle[i].c_str());
  
    
        cpgslw(1);
             
        cpgsch(0.6);
    

	cout << "drawing graph " << i << " ndata=" << nData << endl;

        float *x = new float[nData];
        float *y = new float[nData];

        for (int j = 0; j < nData; j++)
	{
	    x[j] = xdata[i][j];
            y[j] = ydata[i][j];
	

        }

        cpgpt(nData, x, y, 9);
   

        delete [] x;
        delete [] y;

        cpgslw(1);
        cpgsch(0.5);

        cpgsci(1);
    }  // End of this plot


 
  
}







/****************************************************************
 *
 *
 *****************************************************************/
void drawGraph( int ngraphs, 
                vector< vector<float> > &xdata,
                vector< vector<float> > &ydata, 
                vector<string> &subTitle, 
                char *xlabel, 
                char *ylabel, 
                char *title, 
                float x1, 
                float x2,  
                float xmin1, 
                float xmax1, 
                float ymin1, 
                float ymax1, 
                bool equalAxes,
                float dw1)
  
{
    
    
  
    float y1 = 0.1;  
       
    float xmin =  10000.0;
    float xmax = -10000.0;
    float dw = dw1;

    for (int i = 0; i < ngraphs; i++)             
    {
        for (unsigned j = 0; j <  xdata[i].size(); j++)
        {
        
            if (xdata[i][j] < xmin) xmin = xdata[i][j];
            if (xdata[i][j] >  xmax) xmax = xdata[i][j];
         
        }
    }				  

    cpgrnge(xmin, xmax, &xmin, &xmax);

 

    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }

  



    
    int k = 1;
    for (int i = 0; i < ngraphs; i++)                 
    {
        cpgsci(1); 
     

        float ymin = 100000.0;
        float ymax = -ymin;



        for (unsigned j = 0; j <  ydata[i].size(); j++)
        {
        
            if (ydata[i][j] < ymin) ymin = ydata[i][j];
            if (ydata[i][j] > ymax) ymax = ydata[i][j];
         
        }
    				  


        if (fabs(ymax - ymin) < 1e-10) 
        {
            ymin = ymin - 0.1;
            ymax = ymax + 0.1;
        }    


  
        if ( fabs(ymin1 - ymax1) > 1e-6) 
        {
            ymin = ymin1;
            ymax = ymax1;
        }
  

  
        float y11 = y1 + float(k - 1) * dw;
        float y22 = y1 + float(k) * dw;

        if (y22 > 0.9)
	{
	    k = 1;
            y11 = y1;
            y22 = y1 + dw;


            cpgsch(0.8);
            cpgmtxt("T", 1, 0.5, 0.5, title);
 

            cpgpage();
        }
        
	k++;
    

	//        cout << "Line 371 x1=" << x1 << " x2=" << x2 << " y11=" << y11 << " y22=" << y22 << endl;

        cpgsvp(x1, x2, y11, y22);

      
     
      
        if (equalAxes)
        { 
	    cpgwnad(xmin, xmax, ymin, ymax);
        }
        else
        {
            cpgswin(xmin, xmax, ymin, ymax);
        
        }

      
  
        cpgsls(1);


        cpgsch(0.8);
        cpgslw(1);
  

        if (k == 2)
	{
         
            cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
            cpgbox("n", 0.0, 0, "n", 0.0, 0);
            cpgsch(1.0); 
            cpgslw(1);
            cpglab("", ylabel, "");
            cpgmtxt("B", 2.0, 0.5, 0.5, xlabel );

        }
   
        else
	{

            cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);       
            cpgbox("", 0.0, 0, "n", 0.0, 0);
            cpgsch(1.0); 
            cpgslw(1);
            cpglab("", ylabel, "");
          

        }  

    

        cpgsch(0.9);
      
        cpgmtxt("T", -1, 0.5, 0.5, subTitle[i].c_str());
  
    

        cpgslw(1);
  
           
        cpgsch(0.6);
        int nData = xdata[i].size();           
        float *x = new float[nData];
        float *y = new float[nData];

        for (int j = 0; j < nData; j++)
	{
	    x[j] = xdata[i][j];
            y[j] = ydata[i][j];
	

        }

        cpgpt(nData, x, y, 9);
   

        delete [] x;
        delete [] y;

        cpgslw(1);
        cpgsch(0.5);

        cpgsci(1);
    }  // End of this plot


    cpgsch(0.8);
    cpgmtxt("T", 1, 0.5, 0.5, title);
  
}


/****************************************************************
 *
 *
 *****************************************************************/
void drawGraph( int ngraphs, 
                vector< vector<float> > &xdata,
                vector< vector<float> > &ydata,  
                void (*Action)(vector<float> &x, vector<float> &y, void * data),
                vector<void *> &data, 
                vector<string> &subTitle, 
                char *xlabel, 
                char *ylabel, 
                char *title, 
                float x1, 
                float x2,  
                float xmin1, 
                float xmax1, 
                float ymin1, 
                float ymax1, 
                bool equalAxes,
                float dw1)
  
{
    

  
    float y1 = 0.1;  
       
    float xmin =  10000.0;
    float xmax = -10000.0;
    float dw = dw1;

    for (int i = 0; i < ngraphs; i++)             
    {
        for (unsigned j = 0; j <  xdata[i].size(); j++)
        {
        
            if (xdata[i][j] < xmin) xmin = xdata[i][j];
            if (xdata[i][j] >  xmax) xmax = xdata[i][j];
         
        }
    }				  

    cpgrnge(xmin, xmax, &xmin, &xmax);

 

    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }

  



    
    int k = 1;
    for (int i = 0; i < ngraphs; i++)                 
    {

      cout << "Line 1323 i=" << i << endl;
        cpgsci(1); 
     

        float ymin = 100000.0;
        float ymax = -ymin;



        for (unsigned j = 0; j <  ydata[i].size(); j++)
        {
        
            if (ydata[i][j] < ymin) ymin = ydata[i][j];
            if (ydata[i][j] > ymax) ymax = ydata[i][j];
         
        }
    				  
   cout << "Line 1340 i=" << i << endl;
  

        if (fabs(ymax - ymin) < 1e-10) 
        {
            ymin = ymin - 0.1;
            ymax = ymax + 0.1;
        }    


  
        if ( fabs(ymin1 - ymax1) > 1e-6) 
        {
            ymin = ymin1;
            ymax = ymax1;
        }
  

  
        float y11 = y1 + float(k - 1) * dw;
        float y22 = y1 + float(k) * dw;



   cout << "Line 1364 i=" << i << endl;
  


        if (y22 > 0.9)
	{
	    k = 1;
            y11 = y1;
            y22 = y1 + dw;


            cpgsch(0.8);
            cpgmtxt("T", 1, 0.5, 0.5, title);
 

            cpgpage();
        }
        
	k++;
    

	//        cout << "Line 371 x1=" << x1 << " x2=" << x2 << " y11=" << y11 << " y22=" << y22 << endl;

        cpgsvp(x1, x2, y11, y22);

      
     
      
        if (equalAxes)
        { 
	    cpgwnad(xmin, xmax, ymin, ymax);
        }
        else
        {
            cpgswin(xmin, xmax, ymin, ymax);
        
        }

      
  
        cpgsls(1);


        cpgsch(0.8);
        cpgslw(1);
  

        if (k == 2)
	{
         
            cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
            cpgbox("n", 0.0, 0, "n", 0.0, 0);
            cpgsch(1.0); 
            cpgslw(1);
            cpglab("", ylabel, "");
            cpgmtxt("B", 2.0, 0.5, 0.5, xlabel );

        }
   
        else
	{

            cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);       
            cpgbox("", 0.0, 0, "n", 0.0, 0);
            cpgsch(1.0); 
            cpgslw(1);
            cpglab("", ylabel, "");
          

        }  

    

        cpgsch(0.9);
      
        cpgmtxt("T", -1, 0.5, 0.5, subTitle[i].c_str());
  
    

        cpgslw(1);
  




   cout << "Line 1449 i=" << i << endl;
  
           
        cpgsch(0.6);
        int nData = xdata[i].size();           
        float *x = new float[nData];
        float *y = new float[nData];

        for (int j = 0; j < nData; j++)
	{
	    x[j] = xdata[i][j];
            y[j] = ydata[i][j];
	

        }

        cpgpt(nData, x, y, 9);


   cout << "Line 1468 i=" << i << endl;
  

        /**************************************************
	*
        **************************************************/
	cout << "graph.cc line 1457 calling action\n";

        if (Action)
	{
	  if (data[i])
	  {
            cout << "Line 1462 i=" << i << endl; 
            Action(xdata[i], ydata[i], data[i]);
            cout << "Line 1464 ngraphs=" << ngraphs << endl;
          }
        } 

	cout << "Line 1469\n";  
        delete [] x;
        delete [] y;

        cpgslw(1);
        cpgsch(0.5);


	cout << "Line 1477\n";  
 
        cpgsci(1);
    }  // End of this plot


    cpgsch(0.8);
    cpgmtxt("T", 1, 0.5, 0.5, title);
  

	cout << "Line 1487\n";  
 
}






/****************************************************************
 *
 *
 *****************************************************************/
void drawGraph( int ngraphs, 
                vector< vector<float> > &xdata,
                vector< vector<float> > &ydata, 
                vector< vector<float> > &bin,             
                vector< vector<float> > &median, 
                vector< vector<float> > &lower, 
                vector< vector<float> > &upper, 

                vector<string> &subTitle, 
                char *xlabel, 
                char *ylabel, 
                char *title, 
                float x1, 
                float x2,  
                float xmin1, 
                float xmax1, 
                float ymin1, 
                float ymax1, 
                bool equalAxes,
                float dw1)
  
{
    
    
  
    float y1 = 0.1;  
       
    float xmin =  10000.0;
    float xmax = -10000.0;
    float dw = dw1;

    for (int i = 0; i < ngraphs; i++)             
    {
        for (unsigned j = 0; j <  xdata[i].size(); j++)
        {
        
            if (xdata[i][j] < xmin) xmin = xdata[i][j];
            if (xdata[i][j] >  xmax) xmax = xdata[i][j];
         
        }
    }				  

    cpgrnge(xmin, xmax, &xmin, &xmax);

 

    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }

  



    
    int k = 1;
    for (int i = 0; i < ngraphs; i++)                 
    {
        cpgsci(1); 
     

        float ymin = 100000.0;
        float ymax = -ymin;



        for (unsigned j = 0; j <  ydata[i].size(); j++)
        {
        
            if (ydata[i][j] < ymin) ymin = ydata[i][j];
            if (ydata[i][j] > ymax) ymax = ydata[i][j];
         
        }
    				  


        if (fabs(ymax - ymin) < 1e-10) 
        {
            ymin = ymin - 0.1;
            ymax = ymax + 0.1;
        }    


  
        if ( fabs(ymin1 - ymax1) > 1e-6) 
        {
            ymin = ymin1;
            ymax = ymax1;
        }
  

  
        float y11 = y1 + float(k - 1) * dw;
        float y22 = y1 + float(k) * dw;

        if (y22 > 0.9)
	{
	    k = 1;
            y11 = y1;
            y22 = y1 + dw;


            cpgsch(0.8);
            cpgmtxt("T", 1, 0.5, 0.5, title);
 

            cpgpage();
        }
        
	k++;
    

	//        cout << "Line 371 x1=" << x1 << " x2=" << x2 << " y11=" << y11 << " y22=" << y22 << endl;

        cpgsvp(x1, x2, y11, y22);

      
     
      
        if (equalAxes)
        { 
	    cpgwnad(xmin, xmax, ymin, ymax);
        }
        else
        {
            cpgswin(xmin, xmax, ymin, ymax);
        
        }

      
  
        cpgsls(1);


        cpgsch(0.8);
        cpgslw(1);
  

        if (k == 2)
	{
         
            cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
            cpgbox("n", 0.0, 0, "n", 0.0, 0);
            cpgsch(1.0); 
            cpgslw(1);
            cpglab("", ylabel, "");
            cpgmtxt("B", 2.0, 0.5, 0.5, xlabel );

        }
   
        else
	{

            cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);       
            cpgbox("", 0.0, 0, "n", 0.0, 0);
            cpgsch(1.0); 
            cpgslw(1);
            cpglab("", ylabel, "");
          

        }  

    

        cpgsch(0.9);
      
        cpgmtxt("T", -1, 0.5, 0.5, subTitle[i].c_str());
  
    
        cpgslw(1);
  
           
        cpgsch(0.6);
        int nData = xdata[i].size();           
        float *x = new float[nData];
        float *y = new float[nData];

        for (int j = 0; j < nData; j++)
	{
	    x[j] = xdata[i][j];
            y[j] = ydata[i][j];
	

        }

        cpgpt(nData, x, y, 9);
   
        nData = bin[i].size();
	for (int j = 0; j < nData; j++)
	{

            x[j] = bin[i][j];
            y[j] = median[i][j];
	
        }
        cpgsci(2);
        cpgline(nData, x, y);

	for (int j = 0; j < nData; j++)
	{

            y[j] = lower[i][j];
	
        }
        cpgsci(3);
        cpgline(nData, x, y);


	for (int j = 0; j < nData; j++)
	{

            y[j] = upper[i][j];
	
        }
     
        cpgline(nData, x, y);






        delete [] x;
        delete [] y;

        cpgslw(1);
        cpgsch(0.5);

        cpgsci(1);
    }  // End of this plot


    cpgsch(0.8);
    cpgmtxt("T", 1, 0.5, 0.5, title);
  
}



/*****************************************************
* Subroutine Histogram
*
******************************************************/
void Histogram( int ndata, float *data, float binWidth, 
                 char *xLabel, char *title, 
                 float x1, float x2, float y1, float y2, 
                 float xmin, float xmax)
{
    
         
    if (fabs(xmin - xmax) < 1e-6) {
        cout << "Unable to draw histogram", "xmin=xmax\n";
        return;
    }

   

 
    int noBins = int( (xmax - xmin) / binWidth + 0.0001 );

    if (noBins < 3) {
        cout << "omgraph", "No of bins < 3\n";
        return;
    }
  
    

    /******************************
     * Find the maximum bin size
     *******************************/

    int *frequency = new int[noBins];
    for (int i = 0; i < noBins; i++)
        frequency[i] = 0;




    int noLeft = 0, noRight = 0;

    for (int i = 0; i < ndata; i++) 
    {
        int k = int((data[i] - xmin) / binWidth + 0.00000001);       
        if (k < 0)
	{
	    noLeft++;
            k = 0;
        }
        else if (k >= noBins)
	{
	    noRight++;
            k = noBins - 1;
        }
        frequency[k]++;
       
    }
    
    float  yMax = int( float( maxval(frequency, noBins)) * 1.50); 

    delete [] frequency;  
     
    cpgsvp(x1, x2, y1, y2);

    cpgswin(xmin, xmax, 0.0, yMax);
         

    cpgsci(1);     
    cpgsch(1.0); 
    cpgsfs(2);    
    cpgslw(1);    
    cpghist(ndata, data, xmin, xmax, noBins, 3);

    cpgswin(xmin, xmax, 0.0, (yMax / float(ndata)) * 100.0);
  

    cpgsch(0.8);  
 
    
    cpgbox("bctsi", binWidth, 5, "bctsi", 0.0, 0);

    cpgsch(0.6);  
    cpgbox("n", binWidth, 5, "n", 0.0, 0); 
  
    char text[100];
    int k = 1;
    if (noLeft > 0)
    {
        sprintf( text, "No of data points < %.2f=%d", xmin, noLeft);        
        cpgmtxt("T", -float(k), 0.0, 0.0, text);
        k++;
    }

    if (noRight> 0)
    {
        sprintf( text, "No of data points > %.2f=%d", xmax, noRight);        
        cpgmtxt("T", -float(k), 0.0, 0.0, text);
        k++;
    }



    cpgsch(1.0);

    char yLabel[100];

    sprintf( yLabel, "%s, %d, %s", "FREQUENCY (%) (TOTAL=", ndata, ")" );
       
    cpglab("", yLabel, title);
    
    cpgmtxt("B", 2.0, 0.5, 0.5, xLabel);

}




/*****************************************************
 * Subroutine Histogram
 *
 ******************************************************/
void Histogram( vector<float> & data, float binWidth, 
                 char *xLabel, char *title, 
                 float x1, float x2, float y1, float y2, 
                 float xmin, float xmax)
{
    
         
    
    int nData = data.size(); 


    // Sort the data
    sort( data.begin(), data.end() );

    if (fabs(xmin - xmax) < 1e-6) {
        cout << "Unable to draw histogram", "xmin=xmax\n";
        return;
    }

   


 
    int noBins = int( (xmax - xmin) / binWidth + 0.0001 );
    
  
    if (noBins < 3) {
        cout << "omgraph", "No of bins < 3\n";
        return;
    }
  
    

    /******************************
     * Find the maximum bin size
     *******************************/

    int *frequency = new int[noBins];
    for (int i = 0; i < noBins; i++)
        frequency[i] = 0;




    int noLeft = 0, noRight = 0;

    for (int i = 0; i < nData; i++) 
    {
        int k = int((data[i] - xmin) / binWidth + 0.00000001);       
        if (k < 0)
	{
	    noLeft++;
            k = 0;
        }
        else if (k >= noBins)
	{
	    noRight++;
            k = noBins - 1;
        }
        frequency[k]++;
       
    }
    
  


    int maxFreq =  maxval(frequency, noBins);
  
    float yMax = (int)( float(maxFreq) * 1.25); 


   cout << "maxfreq=" << " ymax=" << yMax << endl;

 


    delete [] frequency;  
     
    cpgsvp(x1, x2, y1, y2);

    cpgswin(xmin, xmax, 0.0, yMax);
         

    cpgsci(1);     

    cpgsch(1.0); 
    cpgsfs(2);    
    cpgslw(1);
    float *xData = new float[nData];
    for(int i = 0; i < nData; i++)
        xData[i] = data[i];



  

    

    cpghist(nData,xData, xmin, xmax, noBins, 3);


    delete [] xData;


    cpgswin(xmin, xmax, 0.0, (yMax / float(nData)) * 100.0);
  

    cpgsch(0.8);  
 
    
    cpgbox("bctsi", binWidth, 5, "bctsi", 0.0, 0);

    cpgsch(0.6);  
    cpgbox("n", binWidth, 5, "n", 0.0, 0); 






  
    char text[100];
    int k = 1;
    if (noLeft > 0)
    {
        sprintf( text, "No of data points < %.2f=%*d", xmin, (int(log10((float)noLeft) + 0.00001) + 1), noLeft);        
        cpgmtxt("T", -float(k), 0.0, 0.0, text);
        k++;
    }

    if (noRight> 0)
    {
        sprintf( text, "No of data points > %.2f=%d", xmax, noRight);        
        cpgmtxt("T", -float(k), 0.0, 0.0, text);
        k++;
    }



    sort(data.begin(), data.end());
    // Get some statistics
    float mean = Mean(data);




    float median = Median(data);;
    float p1 = Percentile(data, 0.05);

    float p2 = Percentile(data, 0.95);



    sprintf(text, "Mean=%.2f, Median=%.2f, 95%c confidence interval %.2f to %.2f", mean, median, '%', p1, p2);
    sprintf(text, "Mean=%.2f, Median=%.2f", mean, median);


    cpgmtxt("T", -float(k), 0.0, 0.0, text);
 


    cpgsch(1.0);

    char yLabel[100];

    sprintf( yLabel, "%s %d, %s", "FREQUENCY (%) (TOTAL=", nData, ")" );
       
    cpglab("", yLabel, title);
    
    cpgmtxt("B", 2.0, 0.5, 0.5, xLabel);


   

}


/*****************************************************
* Subroutine Histogram - for 1 or more data sets
*
******************************************************/
void Histogram( vector< vector<float> > & data, float binWidth, 
                vector<string> &heading, char *xLabel, char *title, 
                float x1, float x2, float y1, float y2, 
                float xmin, float xmax)
{
    
         

    int noDataSets = data.size();   // The number of data sets
 

   
    if (fabs(xmin - xmax) < 1e-6) {
        cout << "Unable to draw histogram", "xmin=xmax\n";
        return;
    }

   


    xmin = floor( xmin );
    int noBins = int( (xmax - xmin) / binWidth + 0.0001 );
    
  
    if (noBins < 3) {
        cout << "omgraph", "No of bins < 3\n";
        return;
    }
  
   
    cout << "histogram line 2237\n";

 

    /******************************
     * Find the maximum bin size
     *******************************/

    int *frequency = new int[noBins];
   
  





    float yMax = 0.0, yMax1 = 0.0;

    int noLeft = 0, noRight = 0;
    int nData1 = 0;
    for (int j = 0; j < noDataSets; j++)
    {
     

        int nData = data[j].size(); 

        cout << "Line 1500 dataset " << j << " ndata=" << nData << endl;
        nData1 += nData;
        for (int i = 0; i < noBins; i++)
            frequency[i] = 0;



        for (int i = 0; i < nData; i++) 
        {
            int k = int((data[j][i] - xmin) / binWidth + 0.00000001);       
            if (k < 1)
	    {
	        noLeft++;
              
            }
            else if (k >= noBins)
	    {
	        noRight++;
                
            }
            else
            frequency[k]++;
       
        }
    
  


       int maxFreq =  maxval(frequency, noBins);
       

       float f = (int)( float(maxFreq) * 1.25); 





       if (f > yMax)
       {
           yMax = f;    


  
       }


 
    }

 

 
    yMax1 = 100.0 * yMax / (float)nData1;
  


    delete [] frequency;  
     
    cpgsvp(x1, x2, y1, y2);

    cpgswin(xmin, xmax, 0.0, yMax);
         

    cpgsci(1);     
    cpgsch(1.0); 
    cpgsfs(2);    
    cpgslw(1);





    
    for (int j = 0; j < noDataSets; j++)
    {
        int nData = data[j].size(); 
      

        float *xData = new float[nData];

        for(int i = 0; i < nData; i++)
            xData[i] = data[j][i];
 
        cpgsci(j + 1);    
        if (j == 0) 
            cpghist(nData,xData, xmin, xmax, noBins, 3);
	else
	    cpghist(nData,xData, xmin, xmax, noBins, 3);
   

        delete [] xData;



        cpgsch(0.6);
       
        cpgmtxt("T", float(-(j + 1)), 0.0, 0.0, heading[j].c_str());


    }

    cpgsci(1);



    char text[100];
    int k = 3;
    if (noLeft > 0)
    {
        sprintf( text, "No of data points < %.2f=%*d", xmin, (int(log10((float)noLeft) + 0.00001) + 1), noLeft);        
        cpgmtxt("T", -float(k), 0.0, 0.0, text);
        k++;
    }

    if (noRight> 0)
    {
        sprintf( text, "No of data points > %.2f=%d", xmax, noRight);        
        cpgmtxt("T", -float(k), 0.0, 0.0, text);
        k++;
    }



    cpgswin(xmin, xmax, 0.0, yMax1);
  
    cpgsch(0.8);  
 
    
    cpgbox("bctsi", binWidth, 5, "bctsi", 0.0, 0);

    cpgsch(0.6);  
    cpgbox("n", binWidth, 5, "n", 0.0, 0); 
 
       
  
    char yLabel[100];

    cout << "Line 1635 ndata1=" << nData1 << endl;
   
    sprintf( yLabel, "%s %d, %s", "FREQUENCY (%) (TOTAL=", nData1, ")" );      
    cpgsch(0.8);  
    cpglab("", yLabel, title);
    cpgmtxt("B", 3.0, 0.5, 0.5, xLabel);


  

}








/*****************************************************
* Subroutine Histogram
*
******************************************************/
void Histogram( vector< vector<float> >& data1, float binWidth,
                vector<string> &subTitle, 
		char *xLabel, 
                char *title, 
                 float x1, float x2, 
                 float xmin, float xmax, float dr)
{
        
    int noHists = data1.size(); // The number of histograms






    if (fabs(xmin - xmax) < 1e-6) {
        cout << "Unable to draw histogram", "xmin=xmax\n";
        return;
    }

   
    xmin = floor( xmin );
    int noBins = int( (xmax - xmin) / binWidth + 0.0001 );
    
  
    if (noBins < 3) {
        cout << "omgraph", "No of bins < 3\n";
        return;
    }
  
    float y1 = 0.1;
   

    int l = 1;

    for (int j = 0; j < noHists; j++) // Plot each histogram 
    {


       
        /******************************
        * Find the maximum bin size
        *******************************/
        int nData = data1[j].size();


	cout << "line 1692 ndata=" << nData << "subTitle=" << subTitle[j] << endl; 

        if( nData == 0 ) continue;


        cout << " line 1686 drawing histogram " << j << endl;
        float y11 = y1 + (float)(l - 1) * dr;
           
        float y12 = y1 + (float)(l) * dr;
        if (y12 > 0.9)
	{


            cpgsch(0.8);

            cpgmtxt("T", 1.0, 0.5, 0.5, title);



	    cpgpage();


	    l = 1;
            y11 = y1;
            y12 = y1 + dr;

        }       




        vector<float> data = data1[j];



 

        // Sort the data
        sort( data.begin(), data.end() );

        int *frequency = new int[noBins];
        for (int i = 0; i < noBins; i++)
            frequency[i] = 0;

        int noLeft = 0, noRight = 0;

        for (int i = 0; i < nData; i++) 
        {


	  //	  cout << i << ") data=" << data[i] << endl;



            int k = int((data[i] - xmin) / binWidth + 0.00000001);       
            if (k <= 0)
	    {
	        noLeft++;
                
            }
            else if (k >= noBins)
	    {
	        noRight++;
                
             
            }
            else
                frequency[k - 1]++;
              
       
        }
    
  
        int maxFreq =  maxval(frequency, noBins);
  

        float yMax1 = 0.0;
	float yMax = HistogramMaximum(maxFreq, nData, yMax1); 


	cout << "Line 1087 ndata=" << nData << " maxfreq=" << maxFreq << " yMax=" << yMax << " ymax1=" << yMax1 << endl;        
       

//        float yMax1 = yMax  * (float)nData / (float)(maxFreq * 100);

	cout << "ymax1=" << yMax1 << endl;


//	        int nsub;


//	   yMax = cpgrnd( (float)maxFreq,  &nsub); 

//	   yMax1 = 100.0 * yMax  / (float)nData;




        delete [] frequency;  

     

   
        cpgsvp(x1, x2, y11, y12);

       
   

        cpgswin(xmin, xmax, 0.0, yMax1);
 



        cpgsci(1);     

        cpgsch(1.0); 
        cpgsfs(2);    
        cpgslw(1);
        float *xData = new float[nData];
        for(int i = 0; i < nData; i++)
            xData[i] = data[i];



        cpghist(nData, xData, xmin, xmax, noBins, 3);




        delete [] xData;
        
      



        
        cpgswin(xmin, xmax, 0.0, yMax);

     

        cpgsch(0.6);  
 

        if (l == 1)    
	{
            cpgbox("bctsi", binWidth, 5, "bctsi", 0.0, 0);

            cpgbox("n", binWidth, 5, "n", 0.0, 0); 




   
        }
        else
	{
     
     

            cpgbox("bc", binWidth, 0, "bctsi", 0.0, 0);
            cpgbox("", binWidth, 0, "n", 0.0, 0); 

    
        }
     

     

      
        cpgsch(0.8);

              
        cpglab("", "Frequency (%)", "");
        if (l == 1) 
            cpgmtxt("B", 2.0, 0.5, 0.5, xLabel);


        cpgmtxt("T", -1, 0.5, 0.5, subTitle[j].c_str());


  
        char text[100];
   

        int k = 2;
        if (noLeft > 0)
        {
            sprintf( text, "No of data points < %.2f=%*d", xmin, (int(log10((float)noLeft) + 0.00001) + 1), noLeft);        
            cpgsch(0.6);

            cpgmtxt("T", -float(k), 0.0, 0.0, text);
            k++;
        }

        if (noRight> 0)
        {
            sprintf( text, "No of data points > %.2f=%d", xmax, noRight);        
            cpgsch(0.6);

            cpgmtxt("T", -float(k), 0.0, 0.0, text);
            k++;
        }



        // Get some statistics
        float mean = Mean( data );

        float median = Median( data );

        float p1 = Percentile( data, 0.05);
        float p2 = Percentile( data, 0.95);

        sprintf(text, "Mean=%.2f, Median=%.2f 95%c confidence interval=%.2f to %.2f, N=%d", mean, median, '%', p1, p2, nData);
  sprintf(text, "Mean=%.2f, Median=%.2f, N=%d", mean, median, nData);

        cpgsch(0.6);

        cpgmtxt("T", -float(k), 0.0, 0.0, text);
     

 


        l++; 
    }  // End of this histogram
   
    cpgsch(0.8);

    cpgmtxt("T", 1.0, 0.5, 0.5, title);

}


/*****************************************************
 * 
 *
 ******************************************************/
void TwoDSlice( float *data, int nx, int ny, float *x) 
{



    float *ylim = new float[ nx * ny];
    int coff = 0;
    float bias = 1.0;
    cpghi2d( data, nx, ny, 0, nx - 1, 0, ny - 1, x, coff, bias, true, ylim); 

    delete [] ylim;

}  

/***************************************************************
*
*
****************************************************************/
float Median(vector<float> &data)
{
    float median;
    int nData = data.size();

    if (nData == 0) return 0.0;
    if (nData == 1) return data[0]; 

    int n2 = nData / 2;

    if ( n2 * 2 == nData ) // Even number of points
        median = (data[n2 - 1] + data[n2]) / 2.0;
    else
      median = data[n2];

    return median;
}    
/***************************************************************
*
*
****************************************************************/
float Mean(float *data, long nData)
{

    double mean = 0.0;
    if (nData == 0) return 0.0;

    for( long i = 0; i < nData; i++)
        mean += (double)data[i];

    return (float)(mean / (double)nData);

}

/***************************************************************
*
*
****************************************************************/
float Mean(vector<float> &data)
{
    if (data.size() == 0) return 0.0;


    return (float)(accumulate( data.begin(), data.end(), 0.0 ) / double(data.size() ));


}

/***************************************************************
*
*
****************************************************************/
float Median(float *data, long nData)
	{
    float median;


    if (nData == 0) return 0.0;
    if (nData == 1) return data[0]; 



  
    int n2 = nData / 2;

    if ( n2 * 2 == nData ) // Even number of points
        median = (data[n2 - 1] + data[n2]) / 2.0;
    else
      median = data[n2];

    return median;
}    

/***************************************************************
*
*
****************************************************************/
float Percentile(vector<float> &data, float perCent)
{

    long size = data.size();
    
    long index = (long)(perCent * (float)size + 0.00001) - 1;
    index = max((long)0, index);
    index = min(index, size - 1); 

    //    cout << "n=" << size << " percent=" << perCent << " index=" << index << " percentile=" << data[index] << endl;

    return data[index];

}
/***************************************************************
*
*
****************************************************************/
float Percentile(float *data, long size, float perCent)
{

 
    
    long index = (long)(perCent * (float)size + 0.00001) - 1;
    index = max((long)0, index);
    index = min(index, size - 1); 

    return data[index];

}

/********************************************************************
*
*
********************************************************************/
float HistogramMaximum(int maxFreq, int nData, float &yMax1)
{
    float yMax = 5.0 + 100.0 * (float)maxFreq / (float)nData; 



    if (yMax >= 90.0)
        yMax = 100.0;
    else if (yMax >= 80.0)
        yMax = 90.0;
    else if (yMax >= 70.0)
        yMax = 80.0;
    else if (yMax >= 60.0)
        yMax = 70.0;
    else if (yMax >= 50.0)
        yMax = 60.0;
    else if (yMax >= 45.0)
        yMax = 50.0;  
    else if (yMax >= 40.0)
        yMax = 45.0;
    else if (yMax >= 35.0)
        yMax = 40.0;  
    else if (yMax >= 30.0)
        yMax = 35.0;
    else if (yMax >= 25.0)
        yMax = 30.0;
    else if (yMax >= 20.0)
        yMax = 25.0;
    else if (yMax >= 15.0)
        yMax = 20.0;
    else if (yMax >= 10.0)
        yMax = 15.0;
 
    else
        yMax = 10.0;


    yMax1 = yMax * nData / 100.0; 
    return yMax;
}

/*********************************************************************
*
*
**********************************************************************/
void binnedData(vector<float>  &xData, 
                vector<float>  &yData,
                float xMin, 
                float xMax, 
                float dx,
                vector<float>  &bins,
                vector<float>  &median,
                vector<float>  &lower,
                vector<float>  &upper)
{

    bins.clear();
    lower.clear();
    upper.clear();
    median.clear();
 
 
    cout << "graph.cc line 2721\n";
    unsigned long n = xData.size();
    
    int noBins = (int) ( (xMax - xMin) / dx + 0.0001);

  


    if (noBins < 5) return;


    vector<float> x;
    vector<alist> theList;

    for (int i = 0; i < noBins; i++)
    {
    
      
        float xl = xMin + float(i) * dx;
        float xu = xMin + float(i + 1) * dx;
    
       
     
   
  

      

        for (unsigned long j = 0; j <  n; j++)
	{
	  
   
	    if ( xData[j] > xl && xData[j] <= xu)
	    { 
                alist a( xData[j], yData[j]);
           
		theList.push_back( a );
            }
 
        }

	  
   
        if (theList.size() > 49)
	{



	  //	   sort( theList.begin(), theList.end());
	  
        

	   for (unsigned k = 0; k < theList.size(); k++)
	       x.push_back(theList[k].x);


	   bins.push_back( (xl + xu) / 2.0 );
	



            x.clear();
           
    

	    for (unsigned k = 0; k < theList.size(); k++)
	        x.push_back(theList[k].y);


  	

            sort(x.begin(), x.end());

	    median.push_back( Median(x) );
   
     
            lower.push_back( Percentile(x, 0.05));
       
	 

	    upper.push_back( Percentile(x, 0.95));
            x.clear();

        }
        theList.clear();
     
      

    } 

   
    cout << "Line 2819 leaving binneddata \n";

}
/*******************************************************
*
*******************************************************/   
void action1(vector<float> &xData, vector<float> &yData, void *data1)
{
 
  cout << "graph.cc action1 line 2813\n";

    if ( data1 == 0) return;

    cout << "graph.cc action1 line 2834 xdata.size=" << xData.size() << endl;


    if (xData.size() < 100) 
    { 
      cout << "Line 2835 returning\n";
        return;
    }
  cout << "graph.cc action1 line 2838\n";


    BinData *data = (BinData *)data1;
    
  cout << "graph.cc action1 line 2844\n";



    vector<float>  bin, median, lower, upper;

  cout << "graph.cc action1 line 2824\n";

 


    binnedData(xData, yData, data->xMin, data->xMax, data->dx, bin, median, lower, upper); 

    cout << "Line 2847\n";
 
   unsigned long nData = bin.size();



  
  
  

    if (nData == 0) return;

  
  


    float *x = new float[nData];
    float *y = new float[nData];

  


    if (nData != median.size() || nData != lower.size() || nData != upper.size() )
      {
        cout << "bug in action1 at line 2042\n";



      } 

  


    for (unsigned long j = 0; j < nData; j++)
    {
        x[j] = bin[j];
        y[j] = median[j];
	
    }


    cout << "Line 2852 nData=" << nData << endl;;
  

 

    for (unsigned long j = 0; j < median.size(); j++)
        y[j] = median[j];




  cout << "Line 2872\n";



    cpgsci(2);
    cpgline(nData, x, y);
    cpgmtxt("T", -2.0, 0.0, 0.0, data->label1.c_str());
      
 
    cout << "Line 2866\n";

   
    for (unsigned long j = 0; j < lower.size(); j++)
        y[j] = lower[j];
	
   
    cpgsci(3);

    cpgline(nData, x, y);
    cpgmtxt("T", -3.0, 0.0, 0.0, data->label2.c_str() );
    
  cout << "Line 2893\n";


    for (unsigned long j = 0; j < upper.size(); j++)
        y[j] = upper[j];

    cpgsci(4);
   
    cpgline(nData, x, y);
    cpgmtxt("T", -4.0, 0.0, 0.0, data->label3.c_str() );

     cout << "Line 2904\n";

   
   

    delete [] x;
    delete [] y;





}

/**************************************************
 * Subroutine TransformMatrix
 * Sets up plotting transformation matrix.
 * Added by CB to prevent the code having to be
 * repeated twice in the main program
 * (as in original).
 ***************************************************/
float* TransformMatrix(float xmin, float xbinWidth, 
                      float ymin, float ybinWidth)
    
{
    float *trimage = new float[6];
 


   trimage[0] = xmin - xbinWidth / 2.0;
   //   trimage[0] = xmin;


    trimage[1] = xbinWidth;
    trimage[2] = 0.0;
    trimage[3] = ymin - ybinWidth / 2.0;

    //     trimage[3] = ymin;


    trimage[4] = 0.0;
    trimage[5] = ybinWidth;

    return trimage;
}

/************************************************************************
*
************************************************************************/
void loadDensity( float *a, int nx, int ny, 
                  float xmin, float xmax, 
                  float ymin, float ymax, 
                  vector<float> & xdata, vector<float> & ydata, int ndata, float &fMax)
{
    
    float dx = (xmax - xmin) / (float)nx;
    float dy = (ymax - ymin) / (float)ny;
  

    for (int i = 0; i < nx * ny; i++)
	  a[i] = 0.0;



  

    fMax = 0.0;
    for (int i = 0; i < ndata; i++)
    {
        if ( xdata[i] < xmin || xdata[i] > xmax ||
             ydata[i] < ymin || ydata[i] > ymax) continue;


	int k = (int)( (xdata[i] - xmin) / dx + 0.001) - 1;
	int l = (int)( (ydata[i] - ymin) / dy + 0.001) - 1;
 
        int n = l * ny + k;
        n = max(0, n);
        n = min(n, nx * ny + 1);
        a[n] += 1.0001;
        a[n] = (float)( (int)a[n]);
        fMax = max(fMax, a[n] );
    }

   
    //        for (int i = 0; i < nx * ny; i++)
    //    	  a[i] /= fMax;


  


}
/******************************************************************
 * subroutine SetUpPalette
 * Set a "palette" of colors in the range of color indices used by
 * PGIMAG.
 *******************************************************************/
void SetUpPalette(int type)
{

    float GL[] = { 0.0, 1.0};
    float GR[] = { 0.0, 1.0};
    float GG[] = { 0.0, 1.0};
    float GB[] = { 0.0, 1.0};

    float RL[] = {-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7};
    float RR[] = { 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0};
    float RG[] = { 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0};
    float RB[] = { 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0};

    float HL[] = { 0.0, 0.2, 0.4, 0.6, 1.0};
    float HR[] = { 0.0, 0.5, 1.0, 1.0, 1.0};
    float HG[] = { 0.0, 0.0, 0.5, 1.0, 1.0};
    float HB[] = { 0.0, 0.0, 0.0, 0.3, 1.0};

    float WL[] = { 0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, 0.95, 1.0};
    float WR[] = { 0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0,  1.0, 1.0};
    float WG[] = { 0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  1.0, 1.0};
    float WB[] = { 0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, 0.95, 1.0};

    float AL[] = { 0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5, 
                   0.5, 0.6, 0.6, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0};
    float AR[] = { 0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0, 
                   0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0};
    float AG[] = { 0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8, 
                   0.6, 0.6, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0};
    float AB[] = { 0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9, 
                   0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};

    float TL[] = { 0.0, 0.5, 0.5, 1.0};
    float TR[] = { 0.2, 0.6, 0.6, 1.0};
    float TG[] = { 0.0, 0.0, 0.5, 1.0};
    float TB[] = { 1.0, 0.0, 0.0, 0.0};

   
    switch (type)
    { 
        case (1): //        -- gray scale

	    cpgctab(GL, GR, GG, GB, 2, 1.0, 0.5);  // Were all 1.0, 0.5
            break;
	case (2): //        -- rainbow

	    cpgctab(RL, RR, RG, RB, 9, 1.0, 0.5);
	    break;
	case (3): //        -- heat
	    cpgctab(HL, HR, HG, HB, 5, 1.0, 0.5);
            break; 

	case (4): //        -- weird IRAF
	    cpgctab(WL, WR, WG, WB, 10, 1.0, 0.5);
            break;
	case (5): //        -- AIPS
	    cpgctab(AL, AR, AG, AB, 20, 1.0, 0.5);
            break; 
	case (6): //        -- TJP
	    cpgctab(TL, TR, TG, TB, 4, 1.0, 0.5);
            
        
    }

}
/*************************************************************
!
! Subroutine DrawColourScale
!
! Plot an annotated grey-scale or color wedge parallel to a given axis
! of the the current viewport. This routine is designed to provide a
! brightness/color scale for an image drawn with PGIMAG or PGGRAY.
! The wedge will be drawn with the transfer function set by PGSITF
! and using the color index range set by PGSCIR.
!
!
! Arguments:
!  SIDE   (input)  : The first character must be one of the characters
!                    'B', 'L', 'T', or 'R' signifying the Bottom, Left,
!                    Top, or Right edge of the viewport.
!                    The second character should be 'I' to use PGIMAG
!                    to draw the wedge, or 'G' to use PGGRAY.
!  DISP   (input)  : the displacement of the wedge from the specified
!                    edge of the viewport, measured outwards from the
!                    viewport in units of the character height. Use a
!                    negative value to write inside the viewport, a
!                    positive value to write outside.
!  WIDTH  (input)  : The total width of the wedge including annotation,
!                    in units of the character height.
!  foreGroundValue   (input)  : The value which is to appear with shade
!                    1 ("foreground"). Use the values of minValue and maxValue
!                    that were supplied to PGGRAY or PGIMAG.
!  backGroundValue   (input)  : the value which is to appear with shade
!                    0 ("background").
!  LABEL  (input)  : Optional units label. If no label is required
!                    use ' '.
!
!*************************************************************/
void DrawColourScale(char side[], float disp, float width, char *label, 
                          float foreGround, float backGround)
{


    float xcharSize, ycharSize;  // Stores the width and height of a character

    SetUpColourIndices();

    cpgsci(10);
    cpgsch(0.8);

    cpgwedg( side, disp, width, foreGround, backGround, "");   // was 0.1-1.0

    cpgsch(0.75); //  Set the label size
    cpgqcs(0, &xcharSize, &ycharSize);
    cpgslw(3);

    cpgmtxt(side, disp + width + ycharSize * 100.0, 0.5, 0.5, label);
    cpgslw(1);
}


/****************************************
! SetUpColourIndice
! Colour indices used to plot fits regions
!***************************************/
void SetUpColourIndices( bool display )
{

 

    cpgscr(1,  0.0, 1.0, 1.0);                   // Cyan
    cpgscr(2,  1.0, 0.0, 1.0);                   // Magenta
    cpgscr(3,  1.0, 1.0, 0.0);                   // Yellow
    cpgscr(4,  0.0, 128.0/255.0, 128.0/255.0);   // Dark cyan
    cpgscr(5,  0.0, 0.0, 128.0/255.0);           // Dark blue
    cpgscr(6,  128.0/255.0, 128.0/255.0, 0.0);   // Dark yellow
    cpgscr(7,  0.0, 128.0/255.0, 0.0);           // Dark green
    cpgscr(8,  128.0/255.0, 0.0, 128.0/255.0);   // Dark magenta
    cpgscr(9,  128.0/255.0, 0.0, 0.0);           // Dark red

    if (display)         // Black background
      cpgscr(10, 1.0, 1.0, 1.0);               // White
    else
      cpgscr(10, 0.0, 0.0, 0.0);               // Black
    
    cpgscr(11, 1.0, 0.0, 0.0);                 // Red for background mask

}

/****************************************
! SetUpColourIndice
! Colour indices used to plot fits regions
!***************************************/
void SetUpDefaultColourIndices( )
{

 
    cpgscr(1,  0.0, 0.0, 0.0);                   // black

    cpgscr(2,  1.0, 0.0, 0.0);                   // red
    cpgscr(3,  0.0, 0.0, 1.0);                   // blue
    cpgscr(4,  0.0, 1.0, 0.0);                   // green



    cpgscr(5,  0.0, 1.0, 1.0);                   // Cyan
    cpgscr(6,  1.0, 0.0, 1.0);                   // Magenta
    cpgscr(7,  1.0, 1.0, 0.0);                   // Yellow
    cpgscr(8,  0.0, 128.0/255.0, 128.0/255.0);   // Dark cyan
    cpgscr(9,  0.0, 0.0, 128.0/255.0);           // Dark blue
    cpgscr(10,  128.0/255.0, 128.0/255.0, 0.0);   // Dark yellow
    cpgscr(11,  0.0, 128.0/255.0, 0.0);           // Dark green
    cpgscr(12,  128.0/255.0, 0.0, 128.0/255.0);   // Dark magenta
    cpgscr(13,   128.0/255.0, 0.0, 0.0);           // Dark red

    cpgscr(14,   1.0, 1.0, 1.0);           // white


}
