// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef LEASTSQUARES_H
#define LEASTSQUARES_H
#include <iostream>
#include <cmath>







template <class kind>
class LeastSquares
{
private:


  typedef kind (*Function)(kind x);
  Function function[10];
  double a[10][10];   // The fitted coefficientscounts
  int nord;    // Polynomial order 
  
public:
  LeastSquares(int nData, kind *x, kind *y, int nord);
  void PolynomialFit(int nb, kind *x, kind *y, int nord);
  static void Gauss(int n, double a[][10]);
  static kind Power(kind x, int n);
  kind Evaluate(kind x);
};
template <class kind>
LeastSquares<kind>::LeastSquares(int nData, kind *x, kind *y, int order)
{

    nord = order;
    //    va_list ap;
    //    va_start(ap, order);

    //  int i = 0;
    
        
    //   while ( function[i++] = va_arg(ap, Function)) {

    //   };

    //   va_end(ap);
    PolynomialFit(nData, x, y, nord);
    
};
/**************************************************************************
 *
 *
 **************************************************************************/
template <class kind>
void LeastSquares<kind>::PolynomialFit(int noData, kind *x, kind *y, int nord)
{
    double yy;
    int i, j, k, jj;
    int m = nord + 1;
    
    for (k = 0; k < m; k++) {
        for(j = 0; j <= m; j++) a[k][j] = 0.0;
    }

    for (k = 0; k < m; k++) {
      for (i = 0; i < noData; i++) {
	for (j = 0; j < m; j++) {
            jj = k + j;
            a[k][j] = a[k][j] + pow(x[i], jj);
        }
        yy = pow(x[i], k);
        a[k][m] = a[k][m] + y[i] * yy;
      }
    }   
    
  
    Gauss(m, a);

    //    cout << "The computed coefficients are-\n";
    //    for (i = 0; i < m; i++)
    //      cout << i << " )- " << a[i][m] << endl;
 

};
template <class kind>
void LeastSquares<kind>::Gauss(int n, double a[][10])
{
    int i, ipv, j, jc, jr, k, kc, nv;
    double r, tm, va;
    
    for (i = 0; i < (n - 1); i++) {
        ipv = i;
        for (j = i + 1; j < n; j++)
            if (fabs( a[ipv][i]) < fabs( a[j][i])) ipv = j;
        if (ipv != i) {
	  for (jc = 0; jc < (n + 1); jc++) {
              tm = a[i][jc]; 
              a[i][jc] = a[ipv][jc];
              a[ipv][jc] = tm;
          }
        }

        for (jr = i + 1; jr < n; jr++) {
	  if (a[jr][i] != 0) {
	    if (a[i][i] == 0.0) {
	        cout << "Matrix is singular!\n";
                exit(0);
             }
             r = a[jr][i] / a[i][i];
             for (kc = i + 1; kc < (n + 1); kc++)
                 a[jr][kc] = a[jr][kc] - r * a[i][kc];
          } 
       }
    }
    if (a[n - 1][n - 1] == 0.0) {
        cout << "Matrix is singular!\n";
        exit(0);
    }
    a[n - 1][n] = a[n - 1][n] / a[n - 1][n - 1];

    for (nv = n - 2; nv >= 0; nv--) {
        va = a[nv][n];
        for (k = nv + 1; k < n; k++) va = va - a[nv][k] * a[k][n];
        a[nv][n] = va / a[nv][nv];
    }     

};
template <class kind>
kind LeastSquares<kind>::Power(kind x, int n)
{
    double f = 1.0;
    int i;
    if (n > 0) {
        for (i = 1; i <= n; i++) f *= (double)x;
    }
    return (kind)f;
}
template <class kind>
kind LeastSquares<kind>::Evaluate(kind x)
{
    double result = 0.0;
    for (int k = 0; k <= nord; k++)
        result += a[k][nord + 1] * Power(x, k);



    //    cout << "Least-squares interpolated value=" << result << endl;

    return (kind)result; 


}

#endif




