// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef MODE_H
#define MODE_H
#include<vector>
#include<iostream>
#include <cmath>
#include <algorithm>
#include<numeric>
#include<iomanip>
#include <cstring>










float halfRangeMode(vector<float> & data, float &mean, float &median);
vector<float> maxInterval(vector<float> & data, unsigned long &iStart, float constant = 0.5);





/**********************************************
 * int Locate                                 *
 **********************************************/
template <class kind>
unsigned long Locate(vector<kind> &data, unsigned long& jl = 0, double x)
{
    unsigned long n = data.size();

    if (data[jl] >  x) return jl;

    if (data[n - 1] <  x) return n - 2;


    unsigned jm;
    
    
    unsigned long ju = n - 1;
    


    while (ju - jl > 1) {
      jm = (ju + jl) >> 1;
      if (data[jm] < x)
          jl = jm;
      else
          ju = jm;
    }
    return jl;

};



template <class kind>
kind Median(vector<kind> &data)
{

    long unsigned n = data.size();
    long unsigned n2 = n / 2;
    if ( n2 * 2 == n) // Even
      return (data[n2 - 1] + data[n2]) / 2.0;
    else
      return data[n2 - 1]; 


}

template <class kind>
kind Mean(vector<kind> &data)
{

    return (kind)accumulate( data.begin(), data.end(), 0.0f ) / (double)(data.size()); 



};


template <class kind>
kind StandardDeviation(vector<kind> &data)
{

    unsigned size = data.size();
    if (size < 2) return (kind)0;

    double mean = accumulate( data.begin(), data.end(), 0.0f ) / (double)(data.size()); 

    double sum = 0.0;

    for (unsigned i = 0; i < size; i++)
    {

        double diff = data[i] - mean; 
        sum += diff * diff;

    }

    return sqrt( sum / double(size - 1));

    




};



template <class kind>
kind Variance(kind *data, int size)
{

    

  double mean = Mean(data, size);

    double sum = 0.0;

    for (int i = 0; i < size; i++)
    {

        double diff = data[i] - mean; 
        sum += diff * diff;

    }

    return sum / double(size - 1);

    




};











template <class kind>
kind Median(kind *data, int n)
{

    
    int n2 = n / 2;
    if ( n2 * 2 == n) // Even
      return (data[n2 - 1] + data[n2]) / 2.0;
    else
      return data[n2 - 1]; 


}

template <class kind>
kind Mean(kind *data, int n)
{

    double sum = 0.0;   
    for(int i = 0; i < n; i++)
        sum += data[i];

    return (kind)(sum / (double)n);

};





vector<float> resampledData(vector<float> &data);
vector<float> resampledData1(vector<float> &data);

float modeError(vector<float> &data, float &mode);





#endif



























