// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "plotgraph.h"
//#include "Dal.h"
#include <vector>
//#include <qstring.h>
//#include <qdir.h>
#include "cpgplot.h"
#include <cmath>
#include <sstream>
#include <map>
#include <iomanip>
#include"param.h"
#include <sstream>
#include "graph.h"

// using ifstream constructors.
#include <iostream>
#include <fstream>
using namespace std;


void drawGraph10( int ndata, 
                float* xdata, 
                float *xerrors, 
                float *ydata, 
                float *yerrors,
                char *title, 
                char *xlabel, 
                char *ylabel, 
                float x1, 
                float x2, 
                float y1, 
                float y2, 
                float xmin1 = -1.0, 
                float xmax1 = -1.0, 
                float ymin1 = -1.0, 
                float ymax1 = -1.0, 
                bool equalAxes = false);
 


void Histogram10( int ndata, float *data, float& binWidth, 
                 char *xLabel, char *title, 
		float x1, float x2, float y1, float y2);
    


void loadData( string& fileName, 
               vector<float> &x , vector<float> &y);





/**************************************************************************************
*
*
**************************************************************************************/

void plotgraph()
{



    int ok =  cpgopen("plot.ps/VC");

    if (ok) {

      cout << "Okay\n";

    }

    

    string dataFile = stringParameter("datafile" );


    vector<float> x1, y1;
   

    loadData( dataFile, x1, y1); 
    








/**********************************************************************
*
**********************************************************************/
    drawMap( x1,          
              y1,      
	     (char *)dataFile.c_str(),
              "X (pixels)", 
              "Y (pixels)", 
              0.1, 
              0.9,
              0.1, 
              0.9,             
              0.0, 
              2024.0, 
              0.0, 
              2024.0, 
	      true);
  
    cpgpage();
    








    int n = x1.size();

    float *x = new float[n];
    float *dx = new float[n];
    float *dy = new float[n];


    float *y = new float[n];
  


    for (int i = 0; i < n; i++)
    {
        x[i] = x1[i];
        dx[i] = 0.0;
        y[i] = y1[i];
        dy[i] = 0.0;



    }

    float xmin = minval(x, n);
    float xmax = maxval(x, n);
  

    float ymin = floor(minval(y, n));
    float ymax = ceil(maxval(y, n));
  


    xmin = 10.0;
    xmax = 15.0;   
    ymin = -0.2;
    ymax = 0.2;





    float binWidth = 50.0;
    Histogram10( n, x, binWidth, 
                 "X (pixels)", (char *)dataFile.c_str(), 
	       0.1, 0.9, 0.1, 0.6);


    cpgpage();




    binWidth = 50.0;
    Histogram10( n, y, binWidth, 
                 "Y (pixels)", (char *)dataFile.c_str(), 
	       0.1, 0.9, 0.1, 0.6);



    cpgclos();




    delete [] x;
    delete [] dx;

    delete [] y;
    delete [] dy;

//    return 0;

}

/****************************************************************
 *
 *
 *****************************************************************/
void drawGraph10( int ndata, 
                float* xdata, 
                float *xerrors, 
                float *ydata, 
                float *yerrors,
                char *title, 
                char *xlabel, 
                char *ylabel, 
                float x1, 
                float x2, 
                float y1, 
                float y2, 
                float xmin1, 
                float xmax1, 
                float ymin1, 
                float ymax1, 
                bool equalAxes)
  
{
    
    float *lower = 0, *upper = 0;
    float xmin, xmax, ymin, ymax;
  
       
    xmin =  10000.0;
    xmax = -10000.0;
    ymin = xmin;
    ymax = xmax;
             
    for (int j = 0; j <  ndata; j++)
    {
        
        if (xdata[j] < xmin) xmin = xdata[j];
        if (xdata[j] >  xmax) xmax = xdata[j];
        if (ydata[j] < ymin) ymin = ydata[j];
        if (ydata[j] > ymax) ymax = ydata[j];
    }
				  
    if (fabs(ymax - ymin) < 1e-10) 
    {
        ymin = ymin - 0.1;
        ymax = ymax + 0.1;
    }    

    cpgsci( 1 );
    cpgsvp(x1, x2, y1, y2);

     
    cpgrnge(xmin, xmax, &xmin, &xmax);
    cpgrnge(ymin, ymax, &ymin, &ymax);



    if ( fabs(xmin1 - xmax1) > 1e-6) 
    {
        xmin = xmin1;
        xmax = xmax1;
    }
  
    if ( fabs(ymin1 - ymax1) > 1e-6) 
    {
        ymin = ymin1;
        ymax = ymax1;
    }
  



    
    if (equalAxes)
    { 
	 cpgwnad(xmin, xmax, ymin, ymax);
    }
    else
    {
        cpgswin(xmin, xmax, ymin, ymax);
        
    }
    cpgsch(1.0);
      
  
    cpgsls(1);
    cpgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0);
    cpgsch(0.8);
    cpgslw(1);
    cpgbox("n", 0.0, 0, "n", 0.0, 0);
   





    cpgsch(1.0); 
    cpgslw(1);
    cpglab("", ylabel, title);
    cpgmtxt("B", 2.0, 0.5, 0.5, xlabel );
  

    cpgslw(1);
  
    cpgsch(0.6);        
    cpgpt(ndata, xdata, ydata, 9);
   

       
    lower = new float[ndata];
    upper = new float[ndata];
    
    for (int i = 0; i < ndata; i++)
    { 
        lower[i] = xdata[i] - xerrors[i];
        upper[i] = xdata[i] + xerrors[i];
    }       
    cpgerrx(ndata, lower, upper, ydata, 0);
          

    for (int i = 0; i < ndata; i++)
    { 
        lower[i] = ydata[i] - yerrors[i];
        upper[i] = ydata[i] + yerrors[i];
    }       

           
    cpgerry(ndata, xdata, lower, upper, 0);
    delete [] lower;
    delete [] upper;  

    cpgslw(1);
    cpgsch(0.5);
 
}



/*****************************************************
 * Subroutine Histogram
 *
 ******************************************************/
         
  
void Histogram10( int ndata, float *data, float& binWidth, 
                 char *xLabel, char *title, 
		float x1, float x2, float y1, float y2)
{

    
    float xmin = minval( data, ndata );
    float xmax = maxval( data, ndata );


    if (fabs(xmin - xmax) < 1e-6) {
        cout << "Unable to draw histogram- xmin=xmax\n";
        return;
    }

   


    xmin = floor( xmin );
    int noBins;
    for(int j = 1; j <= 10; j++)
    {
        noBins = int(ceil((float(ceil(xmax)) - xmin) / binWidth));
        if (noBins < 16 && noBins > 5) break;
        if (noBins < 5) 
	   binWidth = binWidth / 2.0;
        else
	   binWidth = binWidth * 2.0;
        
    }
    if (noBins < 3) {
        cout << "omgraph - No of bins < 3\n";
        return;
    }
    xmax = xmin + float(noBins) * binWidth;   
    

    /******************************
     * Find the maximum bin size
     *******************************/

    int *frequency = new int[noBins];
    for (int i = 0; i < noBins; i++)
        frequency[i] = 0;

    for (int i = 0; i < ndata; i++) 
    {
        int k = int((data[i] - xmin) / binWidth + 0.00000001);       
        k = max(0, k);
        k = min(noBins - 1, k);
        frequency[k]++;
       
    }
    
    float  yMax = int( float( maxval(frequency, noBins)) * 1.50); 

    delete [] frequency;  
     
    cpgsvp(x1, x2, y1, y2);

    cpgswin(xmin, xmax, 0.0, yMax);
         

     
    cpgsch(1.0); 
    cpgsfs(2);    
    cpgslw(1);    
    cpghist(ndata, data, xmin, xmax, noBins, 3);

    cpgswin(xmin, xmax, 0.0, (yMax / float(ndata)) * 100.0);
  

    cpgsch(0.8);  
 
    
    cpgbox("bctsi", binWidth, 5, "bctsi", 0.0, 0);

    cpgsch(0.8);  

    if(noBins > 20) binWidth *= 2;
    cpgbox("n", binWidth, 5, "n", 0.0, 0); 
  

    cpgsch(1.0);

    char yLabel[100]; 

    sprintf( yLabel, "%s%5d%s", "FREQUENCY (%) (TOTAL=", ndata, ")" );
       
    cpglab("", yLabel, title);
    
    cpgmtxt("B", 2.0, 0.5, 0.5, xLabel);

}

/****************************************
* loadData                              *
*****************************************/
void loadData( string& fileName, 
               vector<float> &x, vector<float>&y )





{

    ifstream input( fileName.c_str() );


    const int length = 200;


    if (input.bad())
    {
      
      exit(0);
    }
     
    char line[length];    
   
    while ( input )
    {
        input.getline(line, length);


        istringstream buffer(line);

                
 
        cout << line << endl;
        char name[40], text[15];  
        float x1, y1;
        float sig, semia, exposure;
        int i, j;
        double counts;
        int nx, ny;
	//        buffer >> name >> text >> x1 >> y1;
	buffer >> name >> text >> x1 >> y1 >> sig >> semia >> exposure >> i >> j >> counts >> nx >> ny;


        if ( (nx == 1024 && ny == 1024) || (nx ==2048 && ny == 2048) )
	{
        cout << "file=" << name << " text=" << text << " x=" << x1 << " y=" << y1 << endl;

            x.push_back(x1);
            y.push_back(y1);
        } 
  
  
    
    }
    input.close();
   

}
