// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/* --------------------------------------------------- */
/* 2021-08-08 vny: fixing the problem of non-existing WHITE filter and missed AB-magnitudes:
 [1] replacing AB_MAG with CORR_RATE when searching for the filter with maximal count rate;
 [2] replacing AB_MAG with INS_MAG for the filter with maximal magnitude]
 [3] removing the filter WHITE from the analysis
 [4[ replacing the column name SRC_ID with SRCID  
*/
/* --------------------------------------------------- */

#include "Dal.h"
#include <iostream>
#include "sourcelist.h"
#include <qdir.h>
#include <qstring.h>
#include <typeinfo>
#include <vector>
#include "SasDalUtils.h"
#include "dal_utilities.h"
#include "DataSubspace.h"
#include "utility.h"
#include "OmSourceParameters.h"
/********************************************************
 *
 *
 ******************************************************/
int GetNoRows(const string& fileName, const string& tableName) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


    if ( !dataSet->hasBlock( tableName) ) 
    {
        cout << "Table doesn't exist\n";
        dataSetServer->close(dataSet); // Finished with the file
 
        return 0;

    }

    const Table *table = dataSet->table(tableName);
  
    int size = table->rows();
  
    dataSetServer->close(dataSet); // Finished with the file
 
    return size;
}

/********************************************************
 *
 *
 ******************************************************/
float GetAttribute(const string& fileName, const string& tableName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return 0;    


    if (!dataSet->hasBlock(tableName))    
    {
       cout << "Table doesn't exist\n";
       dataSetServer->close(dataSet); // Finished with the file
 
       return 0;

    }

    const Table *table = dataSet->table(tableName);
  
    float value = -1.0;
    if (table->hasAttribute(name))
        value = table->attribute(name)->asReal();


    dataSetServer->close(dataSet); // Finished with the file
  
    return value;    

}
/********************************************************
 *
 *
 ******************************************************/
float GetRealAttribute(const string& fileName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return 0;    


  
    float value = -1.0;

    if (dataSet->hasAttribute(name))
        value = dataSet->attribute(name)->asReal();


    dataSetServer->close(dataSet); // Finished with the file
  
    return value;    

}


/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const string & value) 
{
    // Open the file

  //  cout << "addattribute line 87\n";
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if ( !dataSet )
        return;    

    //    cout << "Line 94 adding attribute " << name << " =" << value << endl;

    *dataSet->addAttribute( name, "x", comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}

/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string &tableName, 
    const string& name, const string &comment, const string &units, const string & value) 
{
    
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if ( !dataSet )
        return;    


    Table *table = dataSet->table(tableName);


    
    cout << units << endl;
    *table->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    
  

}







/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const bool & value) 
{
    // Open the file
  


    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if ( ! dataSet )
      return;    

  
    *dataSet->addAttribute( name, comment ) = value;




    dataSetServer->close(dataSet); // Finished with the file
  
    

}

/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string &tableName, const string& name, 
        const string &units, const string &comment, const bool & value) 
{
    // Open the file
  


    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if ( ! dataSet )
      return;    


    Table *table = dataSet->table(tableName);

       cout << units << endl;

  
   *table->addAttribute( name, comment ) = value;




    dataSetServer->close(dataSet); // Finished with the file
  
    

}



/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const float & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    



    *dataSet->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}
/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string &tableName, const string& name, 
        const string &units, const string &comment, const float & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    

    Table *table = dataSet->table(tableName);

       cout << units << endl;

    *table->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}




/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const double & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    



    *dataSet->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}

/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string &tableName, const string& name, 
        const string &units, const string &comment, const double & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    


   Table *table = dataSet->table(tableName);

   cout << units << endl;


    *table->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}




/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const int & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    



    *dataSet->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}

/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string &tableName, const string& name, 
           const string &units, const string &comment, const int & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    

   Table *table = dataSet->table(tableName);
   cout << units << endl;

    *table->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}







/********************************************************
 *
 *
 ******************************************************/
bool GetBooleanAttribute(const string& fileName, const string& tableName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return 0;    


   if (!dataSet->hasBlock(tableName)) 
   
   {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

    }

    const Table *table = dataSet->table(tableName);
  
    bool value = false;
    if (table->hasAttribute(name))
        value = table->attribute(name)->asBool();


    dataSetServer->close(dataSet); // Finished with the file
  
    return value;    

}


/********************************************************
 *
 *
 ******************************************************/
string GetAttribute(const string& fileName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return string(" ");;    


  
   string value = " ";
   if (dataSet->hasAttribute(name))
     value = (dataSet->attribute(name)->asString()).c_str();



   dataSetServer->close(dataSet); // Finished with the file
  
   return value;    

}
/********************************************************
 *
 *
 ******************************************************/
vector<float> GetAttributes(const string& fileName, const string& tableName, const string name[], int n) 
{

    vector<float> value;
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return value;    


   if (!dataSet->hasBlock(tableName)) 
   {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return value;

    }

   const Table *table = dataSet->table(tableName);
 
   for (int i = 0; i < n; i++)
   { 

       if (table->hasAttribute(name[i]))
	 {
	   float f = table->attribute(name[i])->asReal();
      
           value.push_back(f);
	 }
       else
   value.push_back(-1.0);
	
   }
   dataSetServer->close(dataSet); // Finished with the file
 
  
   return value;    

}

/********************************************************
*
*
******************************************************/
float *LoadColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     
   
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


   if (!dataSet->hasBlock(tableName)) 
      {
      cout << "Table doesn't exist\n";
    dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

      }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    float *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        float *xData = columnData->real32Data();
        data = new float[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return data;
};


/********************************************************
*
*
******************************************************/
double *LoadDoubleColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     
   
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


   if (!dataSet->hasBlock(tableName)) 
      {
      cout << "Table doesn't exist\n";
    dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

      }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    double *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
        data = new double[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return data;
};

/********************************************************
*
*
******************************************************/


double *LoadColumnData1(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     
   
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


   if (!dataSet->hasBlock(tableName)) 
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

      }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    double *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
        data = new double[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
          
    dataSetServer->close(dataSet); // Finished with the file
   
  
    return data;
};

/********************************************************
*
*
******************************************************/

Integer1 *LoadInt1ColumnData(const string& fileName, 
                             const string& tableName, 
                             const string& columnName, 
                             int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);


    if (!dataSet)
   
      {
	cout << "failure to open data set\n";
      return 0;    
      }

         // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return 0;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    Integer1 *data = 0;
    
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();


       if (!columnData) exit(0);
	
   
      
       //        int *xData = columnData->int32Data();
       //    Integer1 *xData = columnData->int8Data();
        int8 *xData = columnData->int8Data();

        data = new Integer1[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = (Integer1)xData[i]; 
       
	}
     }
     
       
    dataSetServer->close(dataSet); // Finished with the file
      

  
    return data;
};

/********************************************************
*
*
******************************************************/

int *LoadIntColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);


    if (!dataSet)
   
      {
	cout << "failure to open data set\n";
      return 0;    
      }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return 0;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    int *data = 0;
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int *xData = columnData->int32Data();
        data = new int[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
       
	}
     }
     
       
    dataSetServer->close(dataSet); // Finished with the file
      

  
    return data;
};

/**************************************************
 *
 *
 *************************************************/
vector<QString> loadDirectoryFiles(char *filter, char c)
{
  
    QDir d;
    

    d.setFilter (QDir::Files | QDir::Hidden |
               QDir::NoSymLinks);
    //    d.setSorting(QDir::Size | QDir::Name); // Sort by name
    d.setSorting(QDir::Name); // Sort by name

    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    vector<QString> fileList;
  
    while ((fi = it.current()) != 0)
    {
  


        QString s = fi->fileName().latin1();

        if (s[0] == c && s.contains(filter) && s.contains("FIT") && !s.contains("new") )
	{


  
            string filter = GetAttribute( s, "FILTER" );


 
            if (filter != "GRISM1" && filter != "GRISM2" && filter != "BLOCKED") 
	    {


	        fileList.push_back(s);
             
            }
        }
        ++it;
    } 
    return fileList;

}

/**************************************************
 *
 *
 *************************************************/
vector<QString> loadDirectoryFiles(char *filter, char* filter1, char c)
{
  
    QDir d;
    

    d.setFilter (QDir::Files | QDir::Hidden |
               QDir::NoSymLinks);
    //    d.setSorting(QDir::Size | QDir::Name); // Sort by name
    d.setSorting(QDir::Name); // Sort by name

    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    vector<QString> fileList;
  
    while ((fi = it.current()) != 0)
    {
  


        QString s = fi->fileName().latin1();

        if (s[0] == c && ( s.contains(filter) || s.contains(filter1) )  && s.contains("FIT") && !s.contains("new") )
	{


  
            string filter = GetAttribute( s, "FILTER" );


 
            if (filter != "GRISM1" && filter != "GRISM2" && filter != "BLOCKED") 
	    {


	        fileList.push_back(s);
           
            }
        }
        ++it;
    } 
    return fileList;

}


/**************************************************
 *
 *
 *************************************************/
vector<QString> loadDirectoryFiles(char *filter)
{
  
    QDir d;
    d.setFilter (QDir::Files | QDir::Hidden |
               QDir::NoSymLinks);
  
    d.setSorting(QDir::Name); // Sort by name

    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    vector<QString> fileList;
  
    while ((fi = it.current()) != 0)
    {
        QString s = fi->fileName().latin1();
    
        if (s.contains(filter))
	{
	    fileList.push_back(s);
            
        }
        ++it;
    } 
 
    return fileList;

}

/***************************************************************
*
****************************************************************/
bool deleteRowFromTable(const string& fileName,
                        const string& tableName,
                        const int rowNumber)
{


    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);


    if (!dataSet)
    {
	cout << "failure to open data set\n";
        return false;    
    }


    if (!dataSet->hasBlock(tableName)) 
   
    {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return false;

    }

    Table *table = dataSet->table(tableName);
  
    table->deleteRows(rowNumber, 1);


    dataSetServer->close(dataSet); // Finished with the file
 
    return true;



}



/******************************************************
*
******************************************************/



void setColumnData( const int size, const ColumnData *columnData, const int8* data, const int iStart)
{
    int8 * r64 = columnData->int8Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}



/******************************************************
*
******************************************************/



void setColumnData( const int size, const ColumnData *columnData, const vector<int8> &data, const int iStart )
{
    int8 * r64 = columnData->int8Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}


/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const int16* data, const int iStart )
{
    int16 * r64 = columnData->int16Data();
    for(int i = 0; i < size; i++)
    {
        if( data[i] == -9999 )
            columnData -> setToNull( i );
 
        else

            r64[iStart + i] = data[i];

    }
}
/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int16> &data, const int iStart)
{
    int16 * r64 = columnData->int16Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];


}
/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const int32* data, const int iStart )
{
    int32 * r64 = columnData->int32Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}

/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int32> & data, const int iStart )
{
    int32 * r64 = columnData->int32Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}

/******************************************************
*
******************************************************/


void setColumnData( const int size, const ColumnData *columnData, const real32* data, const int iStart )
{
    real32 * r32 = columnData->real32Data();
    for(int i = 0; i < size; i++)
    {
        if( abs( data[i] + 9999.0) < 0.5 )
	{
            columnData -> setToNull( iStart + i );
        } 
        else
        { 

            r32[iStart + i] = data[i];

        }

    }
}

/******************************************************
*
******************************************************/


void setColumnData( const int size, const ColumnData *columnData, const vector<real32>& data, const int iStart )
{
    real32 * r32 = columnData->real32Data();
    for(int i = 0; i < size; i++)
        r32[iStart + i] = data[i];

}

/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const real64* data, const int iStart )
{
    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];


}

/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<real64> &data, const int iStart )
{
    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];


}

/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const char* data, const int iStart)
{

  if (!columnData) return;
  //    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
      cout << data[iStart + i] << endl;


}
/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real32> &data)
{
  //cout<<"L1199 columnNAme="<<columnName<<endl;
  //cout<<"L1200 nData="<<nData<<endl;
  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real32, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;


          
    return true;
};


/******************************************************
*
******************************************************/
bool appendToColumn( Table *table, 
                     const char* columnName, 
                     int iStart,   
		     int nData,
                     vector<real32> data)
{
     

  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart );

    delete columnData;

    return true;
};


/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real64>& data)
{
     
  //cout<<"L1262"<<endl;
  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real64, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( (int)size[1], columnData, data);


    delete columnData;


          
    return true;
};

/****************************************************************
 *
 ****************************************************************/
bool appendToColumn(Table *table, 
                     const char* columnName, 
		     int iStart,    
		     int nData,
                     vector<real64> data)
{
     

 
  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart);


    delete columnData;


          
    return true;
};


/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int8> &data)
{
     

  //cout<<"L1328"<<endl;
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int8, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( (int)size[1], columnData, data);


    delete columnData;


          
    return true;
};

/**********************************************************************************
*
**********************************************************************************/
bool appendToColumn(Table *table, 
                     const char* columnName, 
		     int iStart,
		     int nData,
                     vector<int8> data)
{
     

 
  
   
    // Create the column
    Column *column = table->column( columnName );

    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart );


    delete columnData;


          
    return true;
};


/********************************************************
*
*
******************************************************/
using namespace SasDalUtils;  

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int16>& data)
{
     
  //  cout << "Line 1381\n";
  //cout<<"L1395"<<endl;
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int16, comment, units , size);
 
    ColumnData *columnData = column->data();


    //    cout << "addcolumn line 1395\n";

    setColumnData( (int)size[1], columnData, data);

    //   cout << "addcolumn line 1399\n";

    delete columnData;


          
    return true;
};
/********************************************************************
*
*
********************************************************************/
bool appendToColumn(Table *table, 
                     const char* columnName, 
		     int iStart,    
		     int nData,
                     vector<int16> data)
{
     

  

  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart );


    delete columnData;


          
    return true;
};




/********************************************************
*
*
******************************************************/
using namespace SasDalUtils;  

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int32> &data)
{
     
  //cout<<"L1468"<<endl;
  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int32, comment, units , size);
 
    ColumnData *columnData = column->data();

 
    setColumnData( (int)size[1], columnData, data);


    delete columnData;


          
    return true;
};

/***************************************************
*
*
**************************************************/
bool appendToColumn(Table *table, 
                     const char* columnName,                  
		     int iStart,   
		     int nData,
                     vector<int32> data)
{
     

 
  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart );


    delete columnData;


          
    return true;
};




/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName, 
                     const char* tableName, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,
                     int columnWidth,   
		     int nData,
                     vector<string> &data)
{
     
  //cout<<"L1539"<<endl;
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }


    // Read the table
    Table *table = dataSet->table(tableName);
  
    // Get the size of the table
    vector<unsigned long> size(1);

    size[0] = columnWidth;

   
    // Create the column
    Column *column = 0;
    if ( table->hasColumn( columnName )) 
        column = table->column( columnName );
    else
        column = table->addColumn( columnName, Dal::DString, comment, units , size);
    

    if (column )
    { 
        ColumnData *columnData = column->data();

        if (!columnData) exit(0);
	
	//	cout << "Line 1374 ndata=" << nData << endl;
	for (int i = 0; i < nData; i++) 
        {

	  //	  cout << i <<  ") sourcelist.cc line 1378 adding comment " << data[i] << " to source-list " << fileName << endl;
	    setStringColumnRow( columnData, i, data[i] ); 
      
	}
        delete columnData;

     }
    else
      exit(0);     
         
    dataSetServer->close(dataSet); // Finished with the file
    //    cout << "sourcelist.cc line 1389\n";
    return true;
};

/********************************************************
*
*
******************************************************/
bool appendToColumn(const string fileName, 
                     const char* tableName, 
                     const char* columnName, 
                     int columnWidth,
		     int iStart,    
		     int nData,
                     vector<string> data)
{
     

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }


    // Read the table
    Table *table = dataSet->table(tableName);
  
    // Get the size of the table
    vector<unsigned long> size(1);

    size[0] = columnWidth;

   
    // Create the column
    Column *column = table->column( columnName );
 

    if (column )
    { 
        ColumnData *columnData = table->column(columnName)->data();

        if (!columnData) exit(0);
	
        using namespace SasDalUtils;  


	for (int i = 0; i < nData; i++) 
        {

	  setStringColumnRow( columnData, iStart + i, data[i] ); 
      
	}
        delete columnData;

     }
    else
      exit(0);     
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return true;
};

/**********************************************************************
*  Load all the source data
*
**********************************************************************/
bool loadSourceParameters(const string &fileName, const char* tableName,
			  int& size, vector < OmSourceParameter >& parameters, 
			  string &filter1, float &exposure1) 
{
  
  cout << "L1685 fileName=" << fileName << endl;
  int stringLength=fileName.length();
  printf("L1687 stringLength=%d \n", stringLength);
  cout << "L1688 filter1=" << filter1 << " exposure1=" << exposure1 << endl;
  cout << "L1689 tableName=" << tableName << endl;
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);

       cout << "L1691 Loading source parameters from " << fileName << endl;
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
      
        return false;

    }

    /* vny 2021-08-07 in the case of merged sources lists OBSMER or OBSMLI, there is no
     EXPOSURE parameter because each source has different exposures */
    float exposure = 0.0;
    if(dataSet->hasAttribute("EXPOSURE"))
    {
        exposure = dataSet->attribute("EXPOSURE")->asReal();
	cout << "L1690 exposure=" << exposure << endl;
	exposure1=exposure;
    } else {
      cout << "L1692 missing EXPOSURE " << endl;
      exposure1=-1.0;
    }

    /* vny 2021-08-07 in the case of merged sources lists OBSMER or OBSMLI, there is no
     FILTER parameter because each source has different filters */
    
    if(dataSet->hasAttribute("FILTER"))
    {
        filter1 = dataSet->attribute("FILTER")->asString();
    }
    else
      filter1 = "N/A";
    cout << "L1725 filter1=" << filter1 << endl;


    // Open the table
    const Table *table = dataSet->table(tableName);
    cout << "L1730 loaded table " << tableName << endl;
    
    parameters.clear();
 
    size = table->rows();
    cout << "L1736 size=" << size << endl;
    
    if(size == 0)
    {
        dataSetServer->close(dataSet); // Finished with the file
        return true;
    } 
   
 
   
    parameters.resize( size ); 

    for (int i = 0; i < size; i++)
    {
        parameters[i].magnitude = -1.0;
        parameters[i].exposure = exposure;     
    }


    // Load the columns one by one
    /* vny: 2021-08-07 the files OBSMER or OBSMLI contain the source ID column with the name SRCID (and not SRC_ID)
    /* individual source lists do not have column with the name SRC_ID, instead they have SRCNUM*/
    cout << "L1756 loading columns one by one \n";
    /*if (table->hasColumn( "SRC_ID" ))*/
      if (table->hasColumn( "SRC_ID" )) 
    { 
      /*const ColumnData *columnData = table->column( "SRC_ID" )->data();*/
	const ColumnData *columnData = table->column( "SRC_ID" )->data();           
        int32 *xData = columnData->int32Data();
       
	for (int i = 0; i < size; i++) 
        {
	  parameters[i].number = xData[i]; 
	  cout<<"Poin: "<<xData[i]<<endl; 
	}
     }
     

    // ************ XPOS *************************
    /* Individual source lists contain a column XPOS bus OBSMER and OBSMLI do not contain it*/
    if (table->hasColumn( "XPOS" )) 
    { 
        const ColumnData *columnData = table->column( "XPOS" )->data();
	    
       
        real32 *xData = columnData->real32Data();
       
 
	for (int i = 0; i < size; i++) 
        {
	  parameters[i].xcoord = xData[i]; 
	
	}
     
     }
     
  
    // ************ YPOS *************************
    if (table->hasColumn( "YPOS" )) 
    { 
        const ColumnData *columnData = table->column( "YPOS" )->data();
	    
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].ycoord = xData[i]; 
       
	}
     }
     
    // ************ POSERR *************************



    if (table->hasColumn( "POSERR" )) 
    { 
        const ColumnData *columnData = table->column( "POSERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].positionerror = xData[i]; 
       
	}
     }
     
    // ************ RA *************************



    if (table->hasColumn( "RA" )) 
    { 
        const ColumnData *columnData = table->column( "RA" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rightascension = xData[i]; 
       
	}
     }
     

   
    // ************ DEC *************************

    if (table->hasColumn( "DEC" )) 
    { 
        const ColumnData *columnData = table->column( "DEC" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].declination = xData[i]; 
       
	}
     }
     


  





    if (table->hasColumn( "RATE" )) 
    { 
        const ColumnData *columnData = table->column( "RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       

	for (int i = 0; i < size; i++) 
        {

           if ( isNull( xData[i] ))
               parameters[i].rawcountrate = 0.0;
           else
	       parameters[i].rawcountrate = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "RATE_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "RATE_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	      if (isNull( xData[i]))
               parameters[i].rawcountrateerror = 0.0;
           else
	       parameters[i].rawcountrateerror = xData[i]; 

       
	}
     }





    if (table->hasColumn( "BACKGROUND_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "BACKGROUND_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawbackgroundcountrate = xData[i]; 
       
	}
     }
     
     

    if (table->hasColumn( "SIGNIFICANCE" )) 
    { 
        const ColumnData *columnData = table->column( "SIGNIFICANCE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].significance = xData[i]; 
       
	}
     }
     



    if (table->hasColumn( "CORR_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrcountrate = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "CORR_RATE_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_RATE_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrcountrateerror = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "CORR_BACKGROUND_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_BACKGROUND_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrbackgroundcountrate = xData[i]; 
       
	}
     }
     



    if (table->hasColumn( "PSF1_CORR" )) 
    { 
        const ColumnData *columnData = table->column( "PSF1_CORR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].psf1correction = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "PSF2_CORR" )) 
    { 
        const ColumnData *columnData = table->column( "PSF2_CORR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].psf2correction = xData[i]; 
       
	}
     }
     

    cout << "L2077 loading column MAG\n";

    /******************************************************************************
    *
    *
    ********************************************************************************/
    /* vny 2021-08-07 the combined source lists do not have filter WHITE (removing)*/
    vector<string> filter;
    filter.push_back("UVW2");
    filter.push_back("UVW1");
    filter.push_back("UVM2");
    filter.push_back("V");
    filter.push_back("B");
    filter.push_back("U");
    /*filter.push_back("WHITE");*/

    if (table->hasColumn( "MAG" )) 
    { 
        const ColumnData *columnData = table->column( "MAG" )->data();
	   
        real32 *xData = columnData->real32Data();
       
	for (int i = 0; i < size; i++) 
        {
	   parameters[i].magnitude = xData[i]; 
       
	}

        columnData = table->column( "MAGERR" )->data();
	    
        xData = columnData->real32Data();
       
	for (int i = 0; i < size; i++) 
        {
	   parameters[i].magnitudeerror = xData[i]; 
       
	}

    } /* if has column MAG*/

    else 
    {


      /**************************************************************
       * Find column with the maximum magnitude
       ***************************************************************/
      /* vny 2021-08-07 the combined source lists do not have filter WHITE */
      /* vny: 2021-08-08 all the AB-magnitudes in the test case of 0853782401 are negative
         (trying to use count rate) */
        int maxCount = 0;
	real32 maxData = -1000.0; /* vny 2021-08-07 testing max value of the column xData*/
        /*string maxFilter = "WHITE";*/
	string maxFilter = "V";
        for(unsigned i = 0; i < filter.size(); i++)
        {
	  
	  /*string name = filter[i] + "_AB_MAG";*/
	    string name = filter[i] + "_CORR_RATE";
            if (table->hasColumn( name )) 
	    { 
                const ColumnData *columnData = table->column( name )->data();
                real32 *xData = columnData->real32Data();
       
          
                int count = 0;
	        for (int j = 0; j < size; j++) 
                {
		  /* vny 2021-08-07 test*/
		  if (i==0) {
		    if (xData[j] > maxData){
		      maxData = xData[j];
		      
		    }
		  }
		  /* end test*/
		  if (xData[j] > 0.0 && xData[j] < 22.0) 
                    {
		      count++; 
                      
                    }
	        } /*  for j */
		cout << "L2147 count=" << count << " filter=" << filter[i] << endl; 
                if (count > maxCount)
                {
                    maxCount = count;
                    maxFilter = filter[i];

                } 
	    } /* if table*/
	} /* for i*/
	cout << "L2156 maxCount=" << maxCount << " maxFilter=" << maxFilter << endl;
        string name = ""; 
        if (maxFilter == "WHITE")
            name = "WHITE_INS_MAG";
        else
	  /*name = maxFilter + "_AB_MAG";*/
	  name = maxFilter + "_INS_MAG";
        
	cout << "L2163 name =" << name << endl; 
        const ColumnData *columnData = table->column( name )->data();
        real32 *xData = columnData->real32Data();
       

        const ColumnData *columnData1 = table->column( name + "_ERR" )->data();

        real32 *xData1 = columnData1->real32Data();
       

	for (int i = 0; i < size; i++) 
        {
	    if (xData[i] > 0.0 && xData[i] < 22.0) 
            {
                parameters[i].magnitude = xData[i];
                parameters[i].magnitudeerror = xData1[i];
             }
             else
             {
	        parameters[i].magnitude = -1.0;
                parameters[i].magnitudeerror = -1.0;
             } 
	}
	

    }  

    
    if (table->hasColumn( "NOPIXELS" )) 
    { 
        const ColumnData *columnData = table->column( "NOPIXELS" )->data();
	    

	
       
        int32 *xData = columnData->int32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].nosourcepixels = xData[i]; 
       	 
       

	}
     }
     


   
 
     
    if (table->hasColumn( "APERTURE" )) 
    { 
        const ColumnData *columnData = table->column( "APERTURE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].photometryradius = xData[i]; 
       
	}
     }





    if (table->hasColumn( "FWHM_MAJ" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MAJ" )->data();
	    

     

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semia = xData[i]; 
       
	}
    }

    if (table->hasColumn( "FWHM_MAJ_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MAJ_ERR" )->data();
	    

   

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semiaerr = xData[i]; 
       
	}
    }





    if (table->hasColumn( "FWHM_MIN" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MIN" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semib = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "FWHM_MIN_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MIN_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semiberr = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "PA" )) 
    { 
        const ColumnData *columnData = table->column( "PA" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].theta = xData[i]; 
       
	}
     }
     

    if (table->hasColumn( "PA_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "PA_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].thetaerr = xData[i]; 
	         

	}
     }
     

  


    if (table->hasColumn( "EXTENDED_FLAG" )) 
    { 
        const ColumnData *columnData = table->column( "EEXTENDED_FLAG" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].extensionflag = xData[i]; 
	  
        
	}
     }

    else if (table->hasColumn( "EXTENDED_FLAG_MAX" )) 
    { 
        const ColumnData *columnData = table->column( "EXTENDED_FLAG_MAX" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].extensionflag = xData[i]; 
	  
        
	}
     }





     
 
    if (table->hasColumn( "QUALITY_FLAG_COMB" )) 
    { 
        const ColumnData *data = table->column( "QUALITY_FLAG_COMB" )->data();
	int type = data->dataType();
	//        cout << "datatype=" << type << endl;
        switch (type ) 
        {
	    case 2:
	    {     
	        int8 *xData = data->int8Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
       
	        }
                break;
	    }

	    case 4: 
	    {
	            
	        int16 *xData = data->int16Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
       
	        }
                break;
	    }
	    default:
	    {
	        cout << "bug";
	        exit(0);
            } 

        }



     }
    else if (table->hasColumn( "QUALITY_FLAG" )) 
    {
        const ColumnData *data = table->column( "QUALITY_FLAG" )->data();
	int type = data->dataType();
	//        cout << "datatype=" << type << endl;
        switch (type ) 
        {
	    case 2:
	    {     
	        int8 *xData = data->int8Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
       
	        }
                break;
	    }

	    case 4: 
	    {     
	        
	        int16 *xData = data->int16Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
       
	        }
                break;
	    }
	    default:
	    {
	        cout << "bug";
	        exit(0);
            } 

        }



     }
     



    if (table->hasColumn( "CONFUSED_FLAG" )) 
    { 
        const ColumnData *columnData = table->column( "CONFUSED_FLAG" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].confusionflag = xData[i]; 
       
	}



     }
    else if (table->hasColumn( "CONFISED_FLAG_MAX" )) 
    { 
        const ColumnData *columnData = table->column( "CONFUSED_FLAG_MAX" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].confusionflag = xData[i]; 
       
	}



     }

    //cout << "L2533 finishing with the file (sourcelist.cc) \n";        
    dataSetServer->close(dataSet); // Finished with the file


    //   cout << "Loaded sourcelist okay\n";
      





    return true;
} 


/**********************************************************************
*  Load all the source data
*
**********************************************************************/
bool loadSourceParameters1(const string &fileName, const char* tableName,
			  int& size, vector < OmSourceParameter >& parameters, 
			   string &filter, float &exposure) 
                     

{



 
  
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);









    if(dataSet->hasAttribute("FILTER"))
    {
        filter = dataSet->attribute("FILTER")->asString();
    }
    else
      filter = "N/A";




    //       cout << "Line 2425\n";


    
    if (!dataSet->hasBlock(tableName)) 
    {

      //  cout << "Line 2432\n";


        dataSetServer->close(dataSet); // Finished with the file
       
	//  cout << "Line 2437\n";

        return false;

    }

    //  cout << "Line 2430\n";




    // Open the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();

    parameters.empty();
    parameters.resize( size ); 


   
    for(int i = 0; i < size; i++)
      parameters[i].magnitude = -1.0;
 

     



    // Load the columns one by one

  

    // ************ XPOS *************************

    if (table->hasColumn( "XPOS" )) 
    { 
        const ColumnData *columnData = table->column( "XPOS" )->data();
	    


       
        real32 *xData = columnData->real32Data();
       

 
	for (int i = 0; i < size; i++) 
        {
	  parameters[i].xcoord = xData[i]; 
	
      
 
	}
     


     }
     
  
    // ************ YPOS *************************
    if (table->hasColumn( "YPOS" )) 
    { 
        const ColumnData *columnData = table->column( "YPOS" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].ycoord = xData[i]; 
       
	}
     }
     
    // ************ POSERR *************************



    if (table->hasColumn( "POSERR" )) 
    { 
        const ColumnData *columnData = table->column( "POSERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].positionerror = xData[i]; 
       
	}
     }
     
    // ************ RA *************************



    if (table->hasColumn( "RA" )) 
    { 
        const ColumnData *columnData = table->column( "RA" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rightascension = xData[i]; 
       
	}
     }
    else if (table->hasColumn( "CAT_RA" )) 
     { 
        const ColumnData *columnData = table->column( "CAT_RA" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rightascension = xData[i]; 
       
	}
     }

   
    // ************ DEC *************************

    if (table->hasColumn( "DEC" )) 
    { 
        const ColumnData *columnData = table->column( "DEC" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].declination = xData[i]; 
       
	}
     }
     
     else if (table->hasColumn( "CAT_DEC" )) 
     { 
        const ColumnData *columnData = table->column( "CAT_DEC" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].declination = xData[i]; 
       
	}
     }
     



  





    if (table->hasColumn( "RATE" )) 
    { 
        const ColumnData *columnData = table->column( "RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawcountrate = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "RATE_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "RATE_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawcountrateerror = xData[i]; 
       
	}
     }





    if (table->hasColumn( "BACKGROUND_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "BACKGROUND_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawbackgroundcountrate = xData[i]; 
       
	}
     }
     
     

    if (table->hasColumn( "SIGNIFICANCE" )) 
    { 
        const ColumnData *columnData = table->column( "SIGNIFICANCE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].significance = xData[i]; 
       
	}
     }
     



    if (table->hasColumn( "CORR_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrcountrate = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "CORR_RATE_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_RATE_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrcountrateerror = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "CORR_BACKGROUND_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_BACKGROUND_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrbackgroundcountrate = xData[i]; 
       
	}
     }
     



    if (table->hasColumn( "PSF1_CORR" )) 
    { 
        const ColumnData *columnData = table->column( "PSF1_CORR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].psf1correction = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "PSF2_CORR" )) 
    { 
        const ColumnData *columnData = table->column( "PSF2_CORR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].psf2correction = xData[i]; 
       
	}
     }
     
    //   cout << "Line 2796\n";

    if (table->hasColumn( "MAG" )) 
    { 
        const ColumnData *columnData = table->column( "MAG" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].magnitude = xData[i]; 
       
	}


     }


  


    else if (table->hasColumn( "BMAG" )) 
    { 
        const ColumnData *columnData = table->column( "BMAG" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   
	       parameters[i].magnitude = xData[i]; 
           
	}






    }
    else 
    {
        if (table->hasColumn( "V_AB_MAG" )) 
        { 
            const ColumnData *columnData = table->column( "V_AB_MAG" )->data();
	    	
       
            real32 *xData = columnData->real32Data();
       


	    for (int i = 0; i < size; i++) 
            {
	       if (xData[i] > 0.0 && xData[i] < 22.0) 
	           parameters[i].magnitude = xData[i]; 
               else
	         parameters[i].magnitude = -1.0;
	    }

        }
        if (table->hasColumn( "B_AB_MAG" )) 
            { 
                const ColumnData *columnData = table->column( "B_AB_MAG" )->data();
	           
                real32 *xData = columnData->real32Data();
       
             
	        for (int i = 0; i < size; i++) 
            {
	       if (parameters[i].magnitude < 0.0 && xData[i] > 0.0 && xData[i] < 22.0) 
	           parameters[i].magnitude = xData[i]; 
           
	    }
        }
  

        if (table->hasColumn( "U_AB_MAG" )) 
        { 
            const ColumnData *columnData = table->column( "U_AB_MAG" )->data();
	           
            real32 *xData = columnData->real32Data();
       

	    for (int i = 0; i < size; i++) 
            {
	       if (parameters[i].magnitude < 0.0 && xData[i] > 0.0 && xData[i] < 22.0) 
	           parameters[i].magnitude = xData[i]; 
           
	    }
        }
  

        if (table->hasColumn( "UVW1_AB_MAG" )) 
        { 
            const ColumnData *columnData = table->column( "UVW1_AB_MAG" )->data();
	           
            real32 *xData = columnData->real32Data();
       

	    for (int i = 0; i < size; i++) 
            {
	       if (parameters[i].magnitude < 0.0 && xData[i] > 0.0 && xData[i] < 22.0) 
	           parameters[i].magnitude = xData[i]; 
           
	    }
        }
  

        if (table->hasColumn( "UVW2_AB_MAG" )) 
        { 
            const ColumnData *columnData = table->column( "UVW2_AB_MAG" )->data();
	           
            real32 *xData = columnData->real32Data();
       

	    for (int i = 0; i < size; i++) 
            {
	       if (parameters[i].magnitude < 0.0 && xData[i] > 0.0 && xData[i] < 22.0) 
	           parameters[i].magnitude = xData[i]; 
           
	    }
        }
  

        if (table->hasColumn( "UVM2_AB_MAG" )) 
        { 
            const ColumnData *columnData = table->column( "UVM2_AB_MAG" )->data();
	           
            real32 *xData = columnData->real32Data();
       

	    for (int i = 0; i < size; i++) 
            {
	       if (parameters[i].magnitude < 0.0 && xData[i] > 0.0 && xData[i] < 22.0) 
	           parameters[i].magnitude = xData[i]; 
           
	    }
        }
  
    }


    //   cout << "Line 2948\n";




     
    if (table->hasColumn( "MAGERR" )) 
    { 
        const ColumnData *columnData = table->column( "MAGERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].magnitudeerror = xData[i]; 
       
	}
     }
     

    
    if (table->hasColumn( "NOPIXELS" )) 
    { 
        const ColumnData *columnData = table->column( "NOPIXELS" )->data();
	    

	
       
        int32 *xData = columnData->int32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].nosourcepixels = xData[i]; 
       	 
       

	}
     }
     


   
 
     
    if (table->hasColumn( "APERTURE" )) 
    { 
        const ColumnData *columnData = table->column( "APERTURE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].photometryradius = xData[i]; 
       
	}
     }





    if (table->hasColumn( "FWHM_MAJ" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MAJ" )->data();
	    

     

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semia = xData[i]; 
       
	}
    }

    if (table->hasColumn( "FWHM_MAJ_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MAJ_ERR" )->data();
	    

   

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semiaerr = xData[i]; 
       
	}
    }





    if (table->hasColumn( "FWHM_MIN" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MIN" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semib = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "FWHM_MIN_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MIN_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semiberr = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "PA" )) 
    { 
        const ColumnData *columnData = table->column( "PA" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].theta = xData[i]; 
       
	}
     }
     

    if (table->hasColumn( "PA_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "PA_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].thetaerr = xData[i]; 
	         

	}
     }
     

  


    if (table->hasColumn( "EXTENDED_FLAG" )) 
    { 
        const ColumnData *columnData = table->column( "EXTENDED_FLAG" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].extensionflag = xData[i]; 
	  
        
	}
     }
     
 

    if (table->hasColumn( "QUALITY_FLAG" )) 
    { 
        const ColumnData *data = table->column( "QUALITY_FLAG" )->data();

	int type = data->dataType();
        if (type == 2)
	{ 
      
	    int8 *xData = data->int8Data();
      

	    for (int i = 0; i < size; i++) 
            {
	        parameters[i].qualityflag = xData[i]; 
	  
       
	    }
        }
        else
        {
	    
	    int16 *xData = data->int16Data();
      

	    for (int i = 0; i < size; i++) 
            {
	        parameters[i].qualityflag = xData[i]; 
	  
       
	    }
        }



    
     }
     



    if (table->hasColumn( "CONFUSED_FLAG" )) 
    { 
        const ColumnData *columnData = table->column( "CONFUSED_FLAG" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].confusionflag = xData[i]; 
       
	}
     }
            
    dataSetServer->close(dataSet); // Finished with the file



      
    return true;
} 


/**********************************************************************
*
*********************************************************************/
bool upDateSrcListFile(const string &fileName, const char* tableName,
		       vector< OmSourceParameter > & parameters)
{ 
                     


    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }

    // Open the table
    Table *table = dataSet->table(tableName);


    int size = table->rows();


    ColumnData *columnData = table->column( "QFLAG")->data();

	
    int8* data = columnData->int8Data();


    for (int i = 0; i < size; i++) 
    {
        data[i] = parameters[i].qualityflag; 
       
    }
    columnData->release();    



    columnData = table->column( "EFLAG")->data();

	
    data = columnData->int8Data();


    for (int i = 0; i < size; i++) 
    {
        data[i] = parameters[i].extensionflag; 
       
    }
    columnData->release();    



    columnData = table->column( "CFLAG")->data();

	
    data = columnData->int8Data();


    for (int i = 0; i < size; i++) 
    {
        data[i] = parameters[i].confusionflag; 
       
    }
    columnData->release();    



    dataSetServer->close(dataSet); // Finished with the file

  

    return true;
}


/**********************************************************************
*
*
*********************************************************************/
bool loadImage( const string & imageFile, const char *name, int &nx, int &ny, float image [])
{

    
    DataSet* set = dataSetServer->open( imageFile, Dal::Read);
    if ( !set) return false;

    Array* array = set->array( name );

    ArrayData* arraydata = array->data();
    real32* theData = arraydata->real32Data();

    

    longVec arraySize = array->size();
  
    nx = arraySize[1];
    ny = arraySize[0];




    for (int i = 0; i < nx * ny; i++)
        image[i] = theData[i];
    



    dataSetServer->close(set);


 

    return true;



}
/**********************************************************************
*
*
*********************************************************************/
bool loadImage( const string & imageFile, const char *name, int &nx, int &ny, int8 image[])
{

    
    DataSet* set = dataSetServer->open( imageFile, Dal::Read);
    if ( !set) return false;

    Array* array = set->array( name );

    ArrayData* arraydata = array->data();
    int8* theData = arraydata->int8Data();

    

    longVec arraySize = array->size();
  
    nx = arraySize[1];
    ny = arraySize[0];




    for (int i = 0; i < nx * ny; i++)
        image[i] = theData[i];
    



    dataSetServer->close(set);


 

    return true;



}
/**********************************************************************
*
*
*********************************************************************/
bool loadImage( const string & imageFile, const char *name, int &nx, int &ny, bool image[])
{

    
    DataSet* set = dataSetServer->open( imageFile, Dal::Read);
    if ( !set) return false;

    Array* array = set->array( name );

    ArrayData* arraydata = array->data();
    int8* theData = arraydata->int8Data();

    

    longVec arraySize = array->size();
  
    nx = arraySize[1];
    ny = arraySize[0];

    cout << "Line 2263 nx=" << nx << " ny=" << ny << endl;
    for (int i = 0; i < nx * ny; i++)
    {
      if ( theData[i] > 0) 
	image[i] = true;
      else
        image[i] = false;
    
    }

    dataSetServer->close(set);
 
    return true;


}


/**********************************************************************
*
*********************************************************************/
bool createSrcListFile(const string newSrcFile, const string oldSrcFile, vector< OmSourceParameter > & parameters)
{ 
                     
    dataSetServer->copy( oldSrcFile, newSrcFile );

    // Open the file
    DataSet *dataSet = dataSetServer->open(newSrcFile, Dal::Modify);
  
    if ( !dataSet ) return false;
    cout << "line 2121\n";
    dataSet->deleteBlock( "SRCLIST" );

    // Add the table
    Table *table = dataSet->addTable( "SRCLIST", parameters.size() );


   cout << "line 2128\n";

    vector<real32> data;
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].xcoord );


    if ( !addColumn( table, "XPOS", "pixels", "x coordinate",   
			   (int)parameters.size(), data) )
      return false;



    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].ycoord );

    if ( !addColumn( table, "YPOS", "pixels", "y coordinate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();


    vector<real64> data1;
    for (unsigned i = 0; i < parameters.size(); i++)
        data1.push_back( parameters[i].rightascension );


    if ( !addColumn( table, "RA", "degrees", "Right-ascension",   
			   (int)parameters.size(), data1) )
      return false;

    data1.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data1.push_back( parameters[i].declination );


    if ( !addColumn( table, "DEC", "degrees", "Declination",   
			   (int)parameters.size(), data1) )
      return false;

    data1.clear();


    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].positionerror );


    if ( !addColumn( table, "POSERR", "arc secs", "Position error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /**********************************************************************
     * Count rates
     ********************************************************************/

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrate );


    if ( !addColumn( table, "RATE", "counts/sec", "Raw count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrateerror );


    if ( !addColumn( table, "RATE_ERR", "counts/sec", "Raw count-rate error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawbackgroundcountrate );


    if ( !addColumn( table, "BACKGROUND_RATE", "counts/sec", "Raw background count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].significance );


    if ( !addColumn( table, "SIGNIFICANCE", "", "Signa-to-noise",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrate );


    if ( !addColumn( table, "CORR_RATE", "counts/sec", "Corrected count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrateerror );


    if ( !addColumn( table, "CORR_RATE_ERR", "counts/sec", "Corrected count-rate error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrbackgroundcountrate );


    if ( !addColumn( table, "CORR_BACKGROUND_RATE", "counts/sec", "Corrected background count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    /*****************************************************************************
     * Corrections applied
     ****************************************************************************/ 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf1correction );


    if ( !addColumn( table, "PSF1_CORR", "", "PSF1 correction",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf2correction );


    if ( !addColumn( table, "PSF2_CORR", "", "UV PSF2 correction",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /*************************************************************************************
     * Instrumental magnitudes
     ************************************************************************************/
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitude);


    if ( !addColumn( table, "MAG", "mag", "Instrumental magnitude",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitudeerror);


    if ( !addColumn( table, "MAGERR", "mag", "Instrumental magnitude error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    vector<int> data3;
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( parameters[i].nosourcepixels);


    if ( !addColumn( table, "NOPIXELS", "", "No of source pixels (extended sources)",   
			   (int)parameters.size(), data3) )
      return false;


    data3.clear();    

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].photometryradius);


    if ( !addColumn( table, "APERTURE", "unbinned pixels", "Extraction aperture radius (point-sources)",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semia);


    if ( !addColumn( table, "FWHM_MAJ", "arcsec", "FWHM major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiaerr);


    if ( !addColumn( table, "FWHM_MAJ_ERR", "arcsecs", "FWHM major-axis error of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semib);


    if ( !addColumn( table, "FWHM_MIN", "arcsecs", "FWHM minor-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiberr);


    if ( !addColumn( table, "FWHM_MIN_ERR", "arcsecs", "FWHM minor-axis error of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].theta);


    if ( !addColumn( table, "PA", "counts/sec", "Position angle of major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].thetaerr);


    if ( !addColumn( table, "PA_ERR", "counts/sec", "Error in position-angel of major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 




    vector<int8> data4;
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].extensionflag);


    if ( !addColumn( table, "EFLAG", "", "Source extension flag (0-point, 1-extended)",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].qualityflag);


    if ( !addColumn( table, "QFLAG", "", "Quality flag",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].confusionflag);


    if ( !addColumn( table, "CFLAG", "", "Confusion flag",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    //    table->insertRows( (int)parameters.size(), 100);

    dataSetServer->close( dataSet );

   
    return true;
}

/**********************************************************************
*  Appends new rows to the end of the source-list file
*
*********************************************************************/
bool appendToSrcListFile( const string newSrcFile, vector< OmSourceParameter > & parameters )
{ 
             


    // Open the file
    DataSet *dataSet = dataSetServer->open( newSrcFile, Dal::Modify);
  
   
    if ( !dataSet ) return false;
  

    // Update the table
    Table *table = dataSet->table( "SRCLIST", Dal::Modify );
    int iStart = table->rows();


    // Insert new rows at the end of the file
    table->insertRows( iStart, parameters.size() );
 

    vector<int32> data3;
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( iStart + i + 1 );


    if ( !appendToColumn( table, "SRCNUM",  iStart, (int)parameters.size(), data3) )
      return false;

    data3.clear();

  


    vector<real32> data;
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].xcoord );


    if ( !appendToColumn( table, "XPOS",  iStart, (int)parameters.size(), data) )
      return false;



    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].ycoord );

    if ( !appendToColumn( table, "YPOS", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();


        vector<real64> data1;
    //    for (unsigned i = 0; i < parameters.size(); i++)
    //        data1.push_back( parameters[i].rightascension );


    //    if ( !appendToColumn( table, "RA", iStart, (int)parameters.size(), data1) )
    //      return false;

    //    data1.clear();
    //    for (unsigned i = 0; i < parameters.size(); i++)
    //        data1.push_back( parameters[i].declination );


    //    if ( !appendToColumn( table, "DEC", iStart, (int)parameters.size(), data1) )
    //     return false;

	//    data1.clear();


    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].positionerror );


    if ( !appendToColumn( table, "POSERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /**********************************************************************
     * Count rates
     ********************************************************************/

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrate );


    if ( !appendToColumn( table, "RATE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrateerror );


    if ( !appendToColumn( table, "RATE_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawbackgroundcountrate );


    if ( !appendToColumn( table, "BACKGROUND_RATE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].significance );


    if ( !appendToColumn( table, "SIGNIFICANCE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrate );


    if ( !appendToColumn( table, "CORR_RATE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrateerror );


    if ( !appendToColumn( table, "CORR_RATE_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrbackgroundcountrate );


    if ( !appendToColumn( table, "CORR_BACKGROUND_RATE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    /*****************************************************************************
     * Corrections applied
     ****************************************************************************/ 





    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf1correction );


    if ( !appendToColumn( table, "PSF1_CORR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf2correction );


    if ( !appendToColumn( table, "PSF2_CORR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();





    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rightascension );


    if ( !appendToColumn( table, "SPB_COILOSS", iStart, (int)parameters.size(), data) )
        return false;

    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].declination );


    if ( !appendToColumn( table, "BK_COILOSS", iStart, (int)parameters.size(), data) )
        return false;

    data.clear();








 

    /*************************************************************************************
     * Instrumental magnitudes
     ************************************************************************************/
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitude);


    if ( !appendToColumn( table, "MAG", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitudeerror);


    if ( !appendToColumn( table, "MAGERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

   
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( parameters[i].nosourcepixels);


    if ( !appendToColumn( table, "NOPIXELS", iStart, (int)parameters.size(), data3) )
      return false;


    data3.clear();    

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].photometryradius);


    if ( !appendToColumn( table, "APERTURE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semia);


    if ( !appendToColumn( table, "FWHM_MAJ", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiaerr);


    if ( !appendToColumn( table, "FWHM_MAJ_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semib);


    if ( !appendToColumn( table, "FWHM_MIN", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiberr);


    if ( !appendToColumn( table, "FWHM_MIN_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].theta);


    if ( !appendToColumn( table, "PA", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].thetaerr);


    if ( !appendToColumn( table, "PA_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 




    vector<int8> data4;
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].extensionflag);


    if ( !appendToColumn( table, "EFLAG", iStart, (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].qualityflag);


    if ( !appendToColumn( table, "QFLAG", iStart, (int)parameters.size(), data4) )
      return false;


    data4.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].confusionflag);


    if ( !appendToColumn( table, "CFLAG", iStart, (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    dataSetServer->close( dataSet );

   
    return true;
}








/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName,
               const string tableName, 
               const string columnName, 
               const string units,
               const string comment,   
	       int nData,
               vector<int16> &data)
{
     


  //cout<<"L4386"<<endl;
  //cout<<"L4387 "<< " fileName="<< fileName << endl;
  //cout<<"L4388 "<< " columnName="<< columnName << endl;
  //cout<<"L4389 "<< " comment="<< comment << endl;
  //cout<<"L4390 "<< " nData="<< nData << endl;
  // Open the file
  //DataSet *dataSet = dataSetServer->open("test1.fit", Dal::Create);
  //cout<<"L4394"<<endl;
  DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    
  //cout<<"L4396"<<endl;
   
  if (!dataSet->hasBlock(tableName)) 
    {
      dataSetServer->close(dataSet); // Finished with the file
      exit(0); 
      return false;
      
    } else {
    //cout<<"L4403 block name OK "<<endl;
  }
    // Get the size of the table
  //cout<<"L4406"<<endl;
  Table *table = dataSet->table(tableName, Dal::Modify);
  Column *column = 0;
  if(table->hasColumn( columnName ) )
    {
      //cout<<"L4411 column exists OK" << endl;
      column = table->column(columnName);
    }
  else
    {
      //cout<<"L4416 creating the column " << endl;
      // Create the column
      column = table->addColumn( columnName, Column::Int16, comment, units);
      //cout<<"L4419 column created " << endl;
    }
 
  //cout<<"L4422 " << endl;

  column -> nullValue( -9999 );


  //cout<<"L4427 " << endl;


  ColumnData *columnData = column->data();

  //cout<<"L4432 setting column data " << endl;
    

  setColumnData( nData, columnData, data);


  //cout<<"L4438 deleting column Data" << endl;

  delete columnData;

  //cout<<"L4442 closing dataSet" << endl;

  dataSetServer->close(dataSet); // Finished with the file


  //cout<<"L4445 finished addColumn " << endl;
 



          
  return true;
};



/********************************************************
*
*
******************************************************/
bool LoadColumnData(const Table *table,  const string& columnName, vector<int> & data)
{
     
   
  
    int size = table->rows();
   
  
    data.resize( size );

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) return false;
	
     


      
        int *xData = columnData->int32Data();


    


     

	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return true;

};

  



  
  
/********************************************************
*
*
******************************************************/
bool LoadColumnData(const Table *table,  const string& columnName, vector<float> & data)
{
     
   
  
    int size = table->rows();
   
  
    data.resize( size );

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) return false;
	
     


      
        float *xData = columnData->real32Data();


    


     

	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return true;

};

  
    
/********************************************************
*
*
******************************************************/
bool LoadColumnData(const Table *table,  const string& columnName, vector<double> & data)
{
     
  
    int size = table->rows();
   
  
    data.resize( size );

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) return false;
	
     


      
        double *xData = columnData->real64Data();


    


     

	for (int i = 0;  i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return true;

};

  
  














