! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

!*****************************************************************
!
!
!****************************************************************
module statistics_mod
    use param
    use cal
    use dal
 
  use oal
  use caloaldefs
  

  
  use errorHandling
    use sasdal
  
    use omnumerics_mod 
contains
!*****************************************************************
!
!
!****************************************************************
subroutine Statistics(nData, xData, weight, mean, meanStandardError, &
                     standardDeviation, s3, s4, xmin, xmax)
 

    integer, intent(in) :: ndata 
    real(kind=double), intent(in), dimension(ndata) ::  xData, weight
    real(kind=double), intent(out) :: mean
    real(kind=double), intent(out) :: meanStandardError
    real(kind=double), intent(out) :: standardDeviation
    real(kind=double), intent(out) :: xmin, xmax
    real(kind=double), intent(out) ::  s3, s4
    real(kind=double) :: wtsum
  
    integer i, ifail, iwt

    ifail = 1
    iwt = 1  ! No weighting
!    call g01aaf(nData, xData, iwt, weight, mean, standardDeviation, s3, s4, &
!                xmin, xmax, wtsum, ifail)
   
    if (ifail .ne. 0) write(*, *) "Error calling g01aaf!- ifail=", ifail
    write(*, *) "No of data=", ndata
    write(*, *) "Mean=", mean
    meanStandardError = standardDeviation / dsqrt(wtsum)
    write(*, *) "Standard error of mean=", meanStandardError
    write(*, *) "Standard deviation=", standardDeviation
    write(*, *) "Coefficient of skewness=", s3
    write(*, *) "Coefficient of kurtosis=", s4
    
end subroutine Statistics
!************************************
!
! Calculates a 95% confidence interval for mean
!************************************
subroutine MeanConfidenceInterval(ndata, mean, standardDeviation,&
                                  meanLower, meanUpper)
    integer, intent(in) :: ndata
    real(kind=double), intent(in) :: mean, standardDeviation
    real(kind=double), intent(out) :: meanLower, meanUpper
    real(kind=double) :: interval

    interval = 1.96d0 * standardDeviation * standardDeviation / dsqrt(dble(ndata))
    meanLower = mean - interval
    meanUpper = mean + interval
end subroutine MeanConfidenceInterval
!****************************************
! Subroutine MeanTest - Test whether a mean is significantly 
! different from a given value
!*****************************************
subroutine Meantest(ndata, mean, standardDeviation, value, zStatistic)
    integer, intent(in) :: ndata
    real(kind=double), intent(in) :: mean, standardDeviation, value
    real(kind=double), intent(out) :: zStatistic

    zStatistic = (mean - value) / (standardDeviation / dsqrt(dble(ndata)))
end subroutine MeanTest 
!*****************************************************
! Subroutine DrawNormalDistribution
!*****************************************************
subroutine DrawNormalDistribution(noData, binWidth, mean, standardDeviation)
    integer, intent(in) :: noData
    real(kind=double), intent(in) :: mean, standardDeviation
    real(kind=double) :: variance
    real(kind=single), allocatable, dimension(:) :: xData, yData
    integer i
    real(kind=double) xValue, yValue, dx
    real xmin, xmax, ymin, ymax
    integer, parameter :: size = 100
    real(kind=double), intent(in) ::  binWidth
    real(kind=double) ::  constant
    real(kind = double), parameter :: PI = 3.141592653589793_double
    allocate(xData(1 : size))
    allocate(yData(1 : size))

    variance = standardDeviation * standardDeviation
    constant = dble(noData) * binWidth / (standardDeviation * sqrt(2.0_double * PI))
!write(*, *) "DrawNormalDistribution-binWidth=", binWidth, " constant=", constant


    call pgqwin(xmin, xmax, ymin, ymax)
    dx = (xmax - xmin) / real(size - 1)
    do i = 1, size

        xValue = xmin + dx * real(i  - 1)
        yValue = constant * Gaussian(xValue, mean, variance)

        xData(i) = xValue         
        yData(i) = yValue
    end do
    call pgline(size, xData, yData)
    deallocate(xData)
    deallocate(yData)
end subroutine DrawNormalDistribution
!*******************************************
! Function Gaussian
!*******************************************
real(kind=double) function Gaussian(x, mean, variance)
    implicit none
    real(kind=double), intent(in) :: x, mean, variance
    
    Gaussian = exp(-((x - mean) ** 2) / &
               (2.0_double * variance))
end function Gaussian


subroutine kstwo(data1, n1, data2, n2, d, prob)
    real(single), pointer :: data1(:), data2(:)
    integer, intent(in) :: n1, n2
    real(single), intent(out) :: d, prob
    integer :: j1 = 1, j2 = 1
    real(single) :: d1, d2, dt, en1, en2, en, fn1 = 0.0, fn2 = 0.0

    ! Sort the data into increasing order
    call heapSort(data1, n1)
    call heapSort(data2, n2)

    en1 = n1
    en2 = n2
    d = 0.0

    do while (j1 .le. n1 .and. j2 .le. n2)
        d1 = data1(j1)
        d2 = data2(j2)
        if (d1 .le. d2) then
            fn1 = j1 / en1
            j1 = j1 + 1
        end if
        if (d2 .le. d1) then
            fn2 = j2 / en2
            j2 = j2 + 1
        end if
        dt = abs(fn2 - fn1)
        if (d2 .gt. d) d = dt        



   end do
   en = sqrt(en1 * en2 / (en1 + en2))
   prob = probks((en + 0.12 + 0.11 / en) * d)

end subroutine kstwo
real(single) function probks(alam)
    real(single), intent(in) :: alam
    integer :: j
    real(single) :: a2, fac = 2.0, sum = 0.0, term, termbf = 0.0

    a2 = -2.0 * alam * alam
    do j = 1, 100
        term = fac * exp(a2 * j * j)
        sum = sum + term
        if (abs(term) .le. 0.001 * termbf .or. abs(term) .le. 1.0e-8 * sum) then
            probks = sum
            return
        end if
        fac = -fac
        termbf = abs(term)
    end do
    probks =  1.0
end function probks  

end module statistics_mod


























